#include "chesslib.h"

void push(struct move** refHead, int value, int y2, int x2, int y, int x, int type)
{
	struct move* newMove = (struct move*)malloc(sizeof(struct move));
	newMove->value = value;
	newMove->y2 = y2;
	newMove->x2 = x2;
	newMove->y = y;
	newMove->x = x;
	newMove->type = type;
	newMove->next = *refHead;
	*refHead = newMove;
}

void pop(struct move** refHead)
{
	struct move* temp;
	temp = *refHead;
	*refHead = (*refHead)->next;
	free(temp);
}

struct move* getLegalMoves(int turn2, int bCastled, int wCastled, int (*chessboard)[8][8], int (*hasMoved)[2][8], int (*enPassant)[2][8])
{
	/* Loop variables. */
	int i, j, k, l;

	/* List of valid moves. */
	struct move* validMoves;

	validMoves = NULL;

	for(i=0; i<8; i++)
	{
		for(j=0; j<8; j++)
		{
			if((*chessboard)[i][j] > 0)
			{
				for(k=0; k<8; k++)
				{
					for(l=0; l<8; l++)
					{
						if(isValidMove((*chessboard)[i][j], i-4, j-4,
							k-4, l-4, turn2, bCastled, wCastled, chessboard, hasMoved, enPassant))
						{
							push(&validMoves, -10000000, i-4, j-4, k-4, l-4, (*chessboard)[i][j]);
						}
					}
				}
			}
		}
	}

	return validMoves;
}

int moveBRook(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* Loop variable. */
	int i;

	/* The rook is moving vertically.  The following conditions
	 * must be met:
	 * 	- no movement in the horizontal direction
	 * 	- at least one square moved up or down
	 * 	- the desired destination is empty or contains a white piece
	 * 	- all squares between the starting location and the desired
	 * 		destination are empty
	 * 	- it's the black player's turn
	 * 	- this move will not put the black king in check
	 */
	if(x2 == x
		&& y2 != y
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] > B_PAWN)
		&& (chessboard[y2+4][x2+4] == B_ROOK
			|| chessboard[y2+4][x2+4] == B_QUEEN)
		&& !turn2)
	{
		for(i=5+min(y,y2); i<4+max(y,y2); i++)
		{
			if(chessboard[i][x+4] != 0)
			{
				/* We're trying to jump a piece. */
				return 0;
			}
		}
	}
	/* The rook is moving horizontally.  The following conditions
	 * must be met:
	 *      - no movement in the vertical direction
	 *      - at least one square moved left or right
	 *      - the desired destination is empty or contains a white piece
	 *      - all squares between the starting location and the desired
	 *	      destination are empty
	 *      - it's the black player's turn
	 *      - this move will not put the black king in check
	 */
	else if(x2 != x
		&& y2 == y
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] > B_PAWN)
		&& (chessboard[y2+4][x2+4] == B_ROOK
			|| chessboard[y2+4][x2+4] == B_QUEEN)
		&& !turn2)
	{
		for(i=5+min(x,x2); i<4+max(x,x2); i++)
		{
			if(chessboard[y+4][i] != 0)
			{
				/* We're trying to jump a piece. */
				return  0;
			}
		}
	}
	/* The rook isn't moving vertically or horizontlly, so we
	 * have an invalid move.
	 */
	else
	{
		return 0;
	}

	/* We got a legal move, so return 1. */
	return 1;
}

int moveBKnight(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* The knight is trying to move two spaces horizontally and
	 * one space vertically.  The following conditions must be met:
	 * 	- the desired destination is empty or contains a white piece
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	if((abs(x2 - x) == 2)
		&& (abs(y2 - y) == 1)
		&& (chessboard[y+4][x+4] > B_PAWN
			|| chessboard[y+4][x+4] == 0)
		&& chessboard[y2+4][x2+4] == B_KNIGHT
		&& !turn2)
	{
		return 1;
	}
	/* The knight is trying to move two spaces vertically and
	 * one space horizontally.  The following conditions must be met:
	 * 	- the desired destination is empty of contains a white piece
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	else if((abs(x2 - x) == 1)
		&& (abs(y2 - y) == 2)
		&& (chessboard[y+4][x+4] > B_PAWN
			|| chessboard[y+4][x+4] == 0)
		&& chessboard[y2+4][x2+4] == B_KNIGHT
		&& !turn2)
	{
		return 1;
	}

	/* We don't have a valid move, so return 0. */
	return 0;
}

int moveBBishop(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* Loop variable. */
	int i;

	/* The bishop is trying to move the same number of spaces
	 * vertically and horizontally.  The following conditions must
	 * be met:
	 * 	- must move at least one square
	 * 	- desired destination must be empty or contain a white piece
	 * 	- all squares between the starting location and the desired
	 * 		destination are empty
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	if((abs(x2 - x) == abs(y2 - y))
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] > B_PAWN)
		&& (chessboard[y2+4][x2+4] == B_BISHOP
			|| chessboard[y+4][x+4] == B_QUEEN)
		&& !turn2)
	{
		/* Moving down and left. */
		if(x2 > x && y2 > y)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4-i][x2+4-i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving down and right. */
		else if(x2 > x && y > y2)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4+i][x2+4-i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving up and right. */
		else if(x > x2 && y > y2)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4+i][x2+4+i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving up and left. */
		else
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4-i][x2+4+i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
	}
	/* The bishop is not trying to move diagonally, so we have an
	 * illegal move.
	 */
	else
	{
		return 0;
	}

	/* We have a legal move, so return 1. */
	return 1;
}

int moveBQueen(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* The queen moves like the bishop and the rook, though only
	 * in one of these ways each turn.  We can use the logic from
	 * moveBRook() and moveBBishop() to determine if the queen's
	 * move is valid.
	 */
	if((moveBRook(y2, x2, y, x, turn2, chessboard)
		|| moveBBishop(y2, x2, y, x, turn2, chessboard))
		&& chessboard[y2+4][x2+4] == B_QUEEN)
	{
		/* We have a legal move, so return 1. */
		return 1;
	}

	/* We have an illegal move. */
	return 0;
}

int moveBKing(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8])
{
	/* Loop variable. */
	int i;

	/* The king is trying to move one square in any direction.  The
	 * following must be met:
	 * 	- the desired destination is empty or contains a white piece
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	if((abs(x2 - x) <= 1)
		&& (abs(y2 - y) <= 1)
		&& (chessboard[y+4][x+4] > B_PAWN
			|| chessboard[y+4][x+4] == 0)
		&& chessboard[y2+4][x2+4] == B_KING
		&& !turn2)
	{
		/* The move is valid, so return 1. */
		return 1;
	}
	/* The king is trying to castle on the queen's side.  The following
	 * must be met:
	 * 	- neither the king nor the queen's rook have moved previously
	 * 	- all squares between the king and the queen's rook are empty
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	else if((x == -4 && y == 3)
		&& hasMoved[1][4] == 0
		&& hasMoved[1][0] == 0
		&& chessboard[7][4] == B_KING
		&& chessboard[7][0] == B_ROOK
		&& !turn2)
	{
		for(i=1; i<=3; i++)
		{
			if(chessboard[7][i] != 0)
			{
				/* A square between the king and the queen's
				 * rook is not empty, so the move is illegal.
				 */
				return 0;
			}
		}
		for(i=0; i<4; i++)
		{
			if(inCheck(x+6+i, y+4, 0, chessboard, hasMoved, enPassant))
			{
				/* The black king would be in check, so the move
				 * is invalid. */
				return 0;
			}
		}
		/* The move is valid.  Return 2 so to signify a queen's-side
		 * castle.
		 */
		return 2;
	}
	/* The king is trying to castle on his own side.  The following must
	 * be met:
	 * 	- neither the king nor his rook have moved previously
	 * 	- all squares between the king and his rook are empty
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	else if((x == 3 && y == 3)
		&& hasMoved[1][4] == 0
		&& hasMoved[1][7] == 0
		&& chessboard[7][4] == B_KING
		&& chessboard[7][7] == B_ROOK
		&& !turn2)
	{
		for(i=5; i<=6; i++)
		{
			if(chessboard[7][i] != 0)
			{
				/* A square between the king and his rook is
				 * not empty, so the move is illegal.
				 */
				return 0;
			}
		}
		for(i=0; i<3; i++)
		{
			if(inCheck(x+3-i, y+4, 0, chessboard, hasMoved, enPassant))
			{
				/* The black king would be in check, so the move
				 * is invalid.
				 */
				return 0;
			}
		}
		/* The move is valid.  Return 3 to signify a king's-side
		 * castle.
		 */
		return 3;
	}

	/* The move is illegal, so return 0. */
	return 0;
}

int moveBPawn(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*enPassant)[8])
{
	/* The pawn is trying to move forward one square.  The following must
	 * be met:
	 * 	- no movement in the horizontal direction
	 * 	- the desired destination must be empty
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	if(x2 == x
		&& y2 == y+1
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] == 0)
		&& chessboard[y2+4][x2+4] == B_PAWN
		&& !turn2)
	{
		return 1;
	}
	/* The pawn is trying to attack by moving one square digonally
	 * forward.  The following must be met:
	 * 	- the desired destination contains a white piece
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	else if((x == x2+1 || x == x2-1)
		&& y == y2-1
		&& chessboard[y+4][x+4] > B_PAWN
		&& chessboard[y2+4][x2+4] == B_PAWN
		&& !turn2)
	{
		return 1;
	}
	/* The pawn is trying to move forward two squares.  The following
	 * must be met:
	 * 	- the pawn has not moved previously
	 * 	- the two squares the pawn will cross are empty
	 * 	- it is the black player's turn
	 * 	- this move will not put the black king in check
	 */
	else if(x2 == x
		&& y2 == y+2
		&& chessboard[y+4][x+4] == 0
		&& chessboard[y+5][x+4] == 0
		&& y2 == 2
		&& chessboard[y2+4][x2+4] == B_PAWN
		&& !turn2)
	{
		/* The move is valid.  Return 2 to signify that the pawn
		 * is using his special first move.
		 */
		return 2;
	}
	/* The pawn is trying to capture en passant.  The following must
	 * be met:
	 * 	- there is a white pawn immediately to the left or right
	 * 		of the black pawn
	 * 	- the pawn being captured used a double move on his last turn
	 * 	- it is black's turn
	 * 	- this move will not put the black king in check
	 */
	else if((x == x2+1 || x == x2-1)
		&& y == y2-1
		&& chessboard[y+5][x+4] == W_PAWN
		&& enPassant[0][x+4] == 1
		&& chessboard[y2+4][x2+4] == B_PAWN
		&& !turn2)
	{
		/* The move is valid.  Return 3 to signify that the pawn
		 * is capturing en passant.
		 */
		return 3;
	}

	/* The move is illegal, so return 0. */
	return 0;
}

int moveWRook(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* Loop variable. */
	int i;

	/* The rook is moving vertically.  The following conditions
	 * must be met:
	 * 	- no movement in the horizontal direction
	 * 	- at least one square moved up or down
	 * 	- the desired destination is empty or contains a black piece
	 * 	- all squares between the starting location and the desired
	 * 		destination are empty
	 * 	- it's the white player's turn
	 * 	- this move will not put the white king in check
	 */
	if(x2 == x
		&& y2 != y
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] < W_ROOK)
		&& (chessboard[y2+4][x2+4] == W_ROOK
			|| chessboard[y2+4][x2+4] == W_QUEEN)
		&& turn2)
	{
		for(i=5+min(y,y2); i<4+max(y,y2); i++)
		{
			if(chessboard[i][x+4] != 0)
			{
				/* We're trying to jump a piece. */
				return 0;
			}
		}
	}
	/* The rook is moving horizontally.  The following conditions
	 * must be met:
	 *      - no movement in the vertical direction
	 *      - at least one square moved left or right
	 *      - the desired destination is empty or contains a black piece
	 *      - all squares between the starting location and the desired
	 *	      destination are empty
	 *      - it's the white player's turn
	 *      - this move will not put the white king in check
	 */
	else if(x2 != x
		&& y2 == y
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] < W_ROOK)
		&& (chessboard[y2+4][x2+4] == W_ROOK
			|| chessboard[y2+4][x2+4] == W_QUEEN)
		&& turn2)
	{
		for(i=5+min(x,x2); i<4+max(x,x2); i++)
		{
			if(chessboard[y+4][i] != 0)
			{
				/* We're trying to jump a piece. */
				return  0;
			}
		}
	}
	/* The rook isn't moving vertically or horizontlly, so we
	 * have an invalid move.
	 */
	else
	{
		return 0;
	}

	/* We got a legal move, so return 1. */
	return 1;
}

int moveWKnight(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* The knight is trying to move two spaces horizontally and
	 * one space vertically.  The following conditions must be met:
	 * 	- the desired destination is empty or contains a black piece
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	if((abs(x2 - x) == 2)
		&& (abs(y2 - y) == 1)
		&& chessboard[y+4][x+4] < W_ROOK
		&& chessboard[y2+4][x2+4] == W_KNIGHT
		&& turn2)
	{
		return 1;
	}
	/* The knight is trying to move two spaces vertically and
	 * one space horizontally.  The following conditions must be met:
	 * 	- the desired destination is empty of contains a black piece
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if((abs(x2 - x) == 1)
		&& (abs(y2 - y) == 2)
		&& chessboard[y+4][x+4] < W_ROOK
		&& chessboard[y2+4][x2+4] == W_KNIGHT
		&& turn2)
	{
		return 1;
	}

	/* We don't have a valid move, so return 0. */
	return 0;
}

int moveWBishop(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* Loop variable. */
	int i;

	/* The bishop is trying to move the same number of spaces
	 * vertically and horizontally.  The following conditions must
	 * be met:
	 * 	- must move at least one square
	 * 	- desired destination must be empty or contain a black piece
	 * 	- all squares between the starting location and the desired
	 * 		destination are empty
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	if((abs(x2 - x) == abs(y2 - y))
		&& (chessboard[y+4][x+4] == 0
			|| chessboard[y+4][x+4] < W_ROOK)
		&& (chessboard[y2+4][x2+4] == W_BISHOP
			|| chessboard[y2+4][x2+4] == W_QUEEN)
		&& turn2)
	{
		/* Moving down and left. */
		if(x2 > x && y2 > y)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4-i][x2+4-i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving down and right. */
		else if(x2 > x && y > y2)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4+i][x2+4-i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving up and right. */
		else if(x > x2 && y > y2)
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4+i][x2+4+i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
		/* Moving up and left. */
		else
		{
			for(i=1; i<abs(x2-x); i++)
			{
				if(chessboard[y2+4-i][x2+4+i] != 0)
				{
					/* We're trying to jump a piece. */
					return 0;
				}
			}
		}
	}
	/* The bishop is not trying to move diagonally, so we have an
	 * illegal move.
	 */
	else
	{
		return 0;
	}

	/* We have a legal move, so return 1. */
	return 1;
}

int moveWQueen(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8])
{
	/* The queen moves like the bishop and the rook, though only
	 * in one of those ways each turn.  We can use the logic from
	 * moveWRook() and moveWBishop() to determine if the queen's
	 * move is valid.
	 */
	if((moveWRook(y2, x2, y, x, turn2, chessboard)
		|| moveWBishop(y2, x2, y, x, turn2, chessboard))
		&& chessboard[y2+4][x2+4] == W_QUEEN)
	{
		/* We have a legal move, so return 1. */
		return 1;
	}

	/* We have an illegal move. */
	return 0;
}

int moveWKing(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8])
{
	/* Loop variable. */
	int i;

	/* The king is trying to move one square in any direction.  The
	 * following must be met:
	 * 	- the desired destination is empty or contains a black piece
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	if((abs(x2 - x) <= 1)
		&& (abs(y2 - y) <= 1)
		&& (chessboard[y+4][x+4] < W_ROOK
			|| chessboard[y+4][x+4] == 0)
		&& chessboard[y2+4][x2+4] == W_KING
		&& turn2)
	{
		/* The move is valid, so return 1. */
		return 1;
	}
	/* The king is trying to castle on the queen's side.  The following
	 * must be met:
	 * 	- neither the king nor the queen's rook have moved previously
	 * 	- all squares between the king and the queen's rook are empty
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if((x == -4 && y == -4)
		&& hasMoved[0][4] == 0
		&& hasMoved[0][0] == 0
		&& chessboard[0][4] == W_KING
		&& chessboard[0][0] == W_ROOK
		&& turn2)
	{
		for(i=1; i<=3; i++)
		{
			if(chessboard[0][i] != 0)
			{
				/* A square between the king and the queen's
				 * rook is not empty, so the move is illegal.
				 */
				return 0;
			}
		}
		for(i=0; i<4; i++)
		{
			if(inCheck(x+6+i, y+4, 1, chessboard, hasMoved, enPassant))
			{
				/* The white king would be in check, so the move
				 * is invalid.
				 */
				return 0;
			}
		}
		/* The move is valid.  Return 2 to signify a queen's-side
		 * castle.
		 */
		return 2;
	}
	/* The king is trying to castle on his own side.  The following must
	 * be met:
	 * 	- neither the king nor his rook have moved previously
	 * 	- all squares between the king and his rook are empty
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if((x == 3 && y == -4)
		&& hasMoved[0][4] == 0
		&& hasMoved[0][7] == 0
		&& chessboard[0][4] == W_KING
		&& chessboard[0][7] == W_ROOK
		&& turn2)
	{
		for(i=5; i<=6; i++)
		{
			if(chessboard[0][i] != 0)
			{
				/* A square between the king and his rook is
				 * not empty, so the move is illegal.
				 */
				return 0;
			}
		}
		for(i=0; i<3; i++)
		{
			if(inCheck(x+3-i, y+4, 1, chessboard, hasMoved, enPassant))
			{
				/* The white king would be in check, so the move
				 * is invalid.
			 	*/
				return 0;
			}
		}
		/* The move is valid.  Return 3 to signify a king's-side
		 * castle.
		 */
		return 3;
	}

	/* The move is illegal, so return 0. */
	return 0;
}

int moveWPawn(int y2, int x2, int y, int x, int turn2, int (*chessboard)[8], int (*enPassant)[8])
{
	/* The pawn is trying to move forward one square.  The following must
	 * be met:
	 * 	- no movement in the horizontal direction
	 * 	- the desired destination must be empty
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	if(x2 == x
		&& y2 == y-1
		&& chessboard[y+4][x+4] == 0
		&& chessboard[y2+4][x2+4] == W_PAWN
		&& turn2)
	{
		return 1;
	}
	/* The pawn is trying to attack by moving one square digonally
	 * forward.  The following must be met:
	 * 	- the desired destination contains a white piece
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if((x == x2+1 || x == x2-1)
		&& y == y2+1
		&& chessboard[y+4][x+4] > 0
		&& chessboard[y+4][x+4] < W_ROOK
		&& chessboard[y2+4][x2+4] == W_PAWN
		&& turn2)
	{
		return 1;
	}
	/* The pawn is trying to move forward two squares.  The following
	 * must be met:
	 * 	- the pawn has not moved previously
	 * 	- the two squares the pawn will cross are empty
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if(x2 == x
		&& y2 == y-2
		&& chessboard[y+4][x+4] == 0
		&& chessboard[y+3][x+4] == 0
		&& y2 == -3
		&& chessboard[y2+4][x2+4] == W_PAWN
		&& turn2)
	{
		/* The move is valid.  Return 2 to signify that the pawn
		 * is using his special fist move.
		 */
		return 2;
	}
	/* The pawn is trying to capture en passant.  The following must
	 * be met:
	 * 	- there is a black pawn immediately to the left or right
	 * 		of the white pawn
	 * 	- the pawn being captured used a double move on his last turn
	 * 	- it is the white player's turn
	 * 	- this move will not put the white king in check
	 */
	else if((x == x2+1 || x == x2-1)
 		&& y == y2+1
		&& chessboard[y+3][x+4] == B_PAWN
		&& enPassant[1][x+4] == 1
		&& chessboard[y2+4][x2+4] == W_PAWN
		&& turn2)
	{
		/* The move is valid.  Return 3 to signify that the pawn
		 * is capturing en passant.
		 */
		return 3;
	}

	/* The move is illegal, so return 0. */
	return 0;
}

int inCheck(int ky, int kx, int correctTurn, int (*chessboard)[8], int (*hasMoved)[8], int (*enPassant)[8])
{
	/* Loop variables. */
	int i, j;

	/* King's coordinates, converted to GLcoords. */
	int x, y;

	/* Translate chessboard coordinates to GLcoords. */
	y = ky-4;
	x = kx-4;

	/* We want to determine if the white king is in check. */
	if(correctTurn)
	{
		/* Loop through the chessboard, checking each non-zero entry
		 * to see if that piece can move to the king's current
		 * location.  If so, the king is in check.  We only check
		 * black pieces because white pieces cannot threaten their
		 * own king.
		 */
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				switch(chessboard[i][j]) {
				case B_ROOK:
					if(moveBRook(i-4, j-4, x, y, 0, chessboard))
					{
						return 1;
					}
				break;
				case B_KNIGHT:
					if(moveBKnight(i-4, j-4, x, y, 0, chessboard))
					{
						return 1;
					}
				break;
				case B_BISHOP:
					if(moveBBishop(i-4, j-4, x, y, 0, chessboard))
					{
						return 1;
					}
				break;
				case B_QUEEN:
					if(moveBQueen(i-4, j-4, x, y, 0, chessboard))
					{
						return 1;
					}
				break;
				case B_KING:
					if(moveBKing(i-4, j-4, x, y, 0, chessboard, hasMoved, enPassant))
					{
						return 1;
					}
				break;
				case B_PAWN:
					if(moveBPawn(i-4, j-4, x, y, 0, chessboard, enPassant))
					{
						return 1;
					}
				break;
				default:
				break;
				}
			}
		}
	}
	/* We're trying to determine if the black king is in check. */
	else
	{
		/* We follow the procedure described above for checking
		 * the white king.
		 */
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				switch(chessboard[i][j]) {
				case W_ROOK:
					if(moveWRook(i-4, j-4, x, y, 1, chessboard))
					{
						return 1;
					}
				break;
				case W_KNIGHT:
					if(moveWKnight(i-4, j-4, x, y, 1, chessboard))
					{
						return 1;
					}
				break;
				case W_BISHOP:
					if(moveWBishop(i-4, j-4, x, y, 1, chessboard))
					{
						return 1;
					}
				break;
				case W_QUEEN:
					if(moveWQueen(i-4, j-4, x, y, 1, chessboard))
					{
						return 1;
					}
				break;
				case W_KING:
					if(moveWKing(i-4, j-4, x, y, 1, chessboard, hasMoved, enPassant))
					{
						return 1;
					}
				break;
				case W_PAWN:
					if(moveWPawn(i-4, j-4, x, y, 1, chessboard, enPassant))
					{
						return 1;
					}
				break;
				default:
				break;
				}
			}
		}
	}

	/* If we haven't returned yet, the king is not in check. */
	return 0;
}

int makeMove(int y2, int x2, int y, int x, int turn2, int flag, int* bCastled, int* wCastled, int (*hasMoved)[2][8], int(*chessboard)[8][8], int (*enPassant)[2][8])
{
	/* We need this to store the value of a captured piece. */
	int temp2;

	/* If any flags were passed to makeMove(), do what must be done
	 * before making the move.
	 */
	switch(flag) {
	case 2:
		(*hasMoved)[!turn2][x2+4] = 1;
	break;
	case 3:
		(*hasMoved)[!turn2][4] = 1;
	break;
	case 4:
		if(turn2)
		{
			(*chessboard)[y+4][x+4] = B_PAWN;
			(*chessboard)[y+3][x+4] = 0;
		}
		else
		{
			(*chessboard)[y+4][x+4] = W_PAWN;
			(*chessboard)[y+5][x+4] = 0;
		}
	break;
	case 5:
		if(turn2)
		{
			(*hasMoved)[0][0] = 1;
			(*hasMoved)[0][4] = 1;
			(*chessboard)[0][0] = 0;
			(*chessboard)[0][4] = 0;
			(*chessboard)[0][2] = W_KING;
			(*chessboard)[0][3] = W_ROOK;
			*wCastled = 1;
		}
		else
		{
			(*hasMoved)[1][0] = 1;
			(*hasMoved)[1][4] = 1;
			(*chessboard)[7][0] = 0;
			(*chessboard)[7][4] = 0;
			(*chessboard)[7][2] = B_KING;
			(*chessboard)[7][3] = B_ROOK;
			*bCastled = 1;
		}
	break;
	case 6:
		if(turn2)
		{
			(*hasMoved)[0][4] = 1;
			(*hasMoved)[0][7] = 1;
			(*chessboard)[0][4] = 0;
			(*chessboard)[0][7] = 0;
			(*chessboard)[0][5] = W_ROOK;
			(*chessboard)[0][6] = W_KING;
			*wCastled = 1;
		}
		else
		{
			(*hasMoved)[1][4] = 1;
			(*hasMoved)[1][7] = 1;
			(*chessboard)[7][4] = 0;
			(*chessboard)[7][7] = 0;
			(*chessboard)[7][5] = B_ROOK;
			(*chessboard)[7][6] = B_KING;
			*bCastled = 1;
		}
	break;
	case 7:
		(*enPassant)[!turn2][x2+4] = 1;
	break;
	default:
	break;
	}

	temp2 = (*chessboard)[y+4][x+4];
	(*chessboard)[y+4][x+4] = (*chessboard)[y2+4][x2+4];
	(*chessboard)[y2+4][x2+4] = 0;

	return temp2;
}

void unmakeMove(int y2, int x2, int y, int x, int turn2, int flag, int pieceHolder, int* bCastled, int* wCastled, int (*hasMoved)[2][8], int (*chessboard)[8][8], int (*enPassant)[2][8])
{
	switch(flag) {
	case 2:
		(*hasMoved)[!turn2][x2+4] = 0;
	break;
	case 3:
		(*hasMoved)[!turn2][4] = 0;
	break;
	case 4:
		if(turn2)
		{
			(*chessboard)[y+3][x+4] = B_PAWN;
		}
		else
		{
			(*chessboard)[y+5][x+4] = W_PAWN;
		}
		pieceHolder = 0;
	break;
	case 5:
		if(turn2)
		{
			(*hasMoved)[0][0] = 0;
			(*hasMoved)[0][4] = 0;
			(*chessboard)[0][0] = W_ROOK;
			(*chessboard)[0][4] = W_KING;
			(*chessboard)[0][2] = 0;
			(*chessboard)[0][3] = 0;
			*wCastled = 0;
		}
		else
		{
			(*hasMoved)[1][0] = 0;
			(*hasMoved)[1][4] = 0;
			(*chessboard)[7][0] = B_ROOK;
			(*chessboard)[7][4] = B_KING;
			(*chessboard)[7][2] = 0;
			(*chessboard)[7][3] = 0;
			*bCastled = 0;
		}
	break;
	case 6:
		if(turn2)
		{
			(*hasMoved)[0][4] = 0;
			(*hasMoved)[0][7] = 0;
			(*chessboard)[0][4] = W_KING;
			(*chessboard)[0][7] = W_ROOK;
			(*chessboard)[0][5] = 0;
			(*chessboard)[0][6] = 0;
			*wCastled = 0;
		}
		else
		{
			(*hasMoved)[1][4] = 0;
			(*hasMoved)[1][7] = 0;
			(*chessboard)[7][4] = 5;
			(*chessboard)[7][7] = 1;
			(*chessboard)[7][5] = 0;
			(*chessboard)[7][6] = 0;
			*bCastled = 0;
		}
	break;
	case 7:
		(*enPassant)[!turn2][x2+4] = 0;
	default:
	break;
	}

	if(flag != 5 && flag != 6)
	{
		(*chessboard)[y2+4][x2+4] = (*chessboard)[y+4][x+4];
		(*chessboard)[y+4][x+4] = pieceHolder;
	}

	return;
}

int isValidMove(int type, int y2, int x2, int y, int x, int turn2, int bCastled, int wCastled, int (*chessboard)[8][8], int (*hasMoved)[2][8], int (*enPassant)[2][8])
{
	/* Loop variables. */
	int i, j;

	/* Temp stores our return value.  Typically, this is 0 if
	 * the move in question is illegal and 1 otherwise.  In
	 * some circumstances, it may be greater.
	 * 	0: illegal move
	 * 	1: legal move
	 * 	2: legal rook move
	 * 	3: legal king move
	 * 	4: legal enPassant
	 * 	5: legal queen's-side castle
	 * 	6: legal king's-side castle
	 * 	7: legal double pawn move
	 */
	int temp;

	/* Lets inner for loops tell outer for loops to break. */
	int found;

	/* If we're pretending to capture a piece, store its
	 * value here for later retrieval.
	 */
	int pieceHolder;

	/* Make sure temp and placeHolder are initialized to
	 * zero to prevent error. */
	temp = 0;
	pieceHolder = 0;

	/* Quick check to prevent moving to your own square or
	 * moving a piece of type 0 (empty square). */
	if((y2 == y && x2 == x) || type == 0)
	{
		return 0;
	}

	/* Determine what type of piece the player selected.
	 * Then, determine whether or not the move is valid
	 * and set temp if it is.
	 */
	switch(type) {
	case B_ROOK:
		if(moveBRook(y2, x2, y, x, turn2, *chessboard))
		{
			temp = 2;
		}
	break;
	case B_KNIGHT:
		temp = moveBKnight(y2, x2, y, x, turn2, *chessboard);
	break;
	case B_BISHOP:
		temp = moveBBishop(y2, x2, y, x, turn2, *chessboard);
	break;
	case B_QUEEN:
		temp = moveBQueen(y2, x2, y, x, turn2, *chessboard);
	break;
	case B_KING:
		switch(moveBKing(y2, x2, y, x, turn2, *chessboard, *hasMoved, *enPassant)) {
		case 1:
			temp = 3;
		break;
		case 2:
			temp = 5;
		break;
		case 3:
			temp = 6;
		break;
		default:
			temp = 0;
		break;
		}
	break;
	case B_PAWN:
		switch((moveBPawn(y2, x2, y, x, turn2, *chessboard, *enPassant))) {
		case 1:
			temp = 1;
		break;
		case 2:
			temp = 7;
		break;
		case 3:
			temp = 4;
		break;
		default:
			temp = 0;
		break;
		}
	break;
	case W_ROOK:
		if(moveWRook(y2, x2, y, x, turn2, *chessboard))
		{
			temp = 2;
		}
	break;
	case W_KNIGHT:
		temp = moveWKnight(y2, x2, y, x, turn2, *chessboard);
	break;
	case W_BISHOP:
		temp = moveWBishop(y2, x2, y, x, turn2, *chessboard);
	break;
	case W_QUEEN:
		temp = moveWQueen(y2, x2, y, x, turn2, *chessboard);
	break;
	case W_KING:
		switch(moveWKing(y2, x2, y, x, turn2, *chessboard, *hasMoved, *enPassant)) {
		case 1:
			temp = 3;
		break;
		case 2:
			temp = 5;
		break;
		case 3:
			temp = 6;
		break;
		default:
			temp = 0;
		break;
		}
	break;
	case W_PAWN:
		switch(moveWPawn(y2, x2, y, x, turn2, *chessboard, *enPassant)) {
		case 1:
			temp = 1;
		break;
		case 2:
			temp = 7;
		break;
		case 3:
			temp = 4;
		break;
		default:
			temp = 0;
		break;
		}
	break;
	default:
		temp = 0;
	break;
	}

	/* Now, make the desired move. If the move wasn't valid,
	 * temp will be 0 and we can just return here.
	 */
	if(temp == 0)
	{
		return 0;
	}

	pieceHolder = makeMove(y2, x2, y, x, turn2, temp, &bCastled, &wCastled, hasMoved, chessboard, enPassant);

	/* Find the king and determine whether or not he
	 * is in check.  If so, the move is not actually
	 * valid.
	 */
	found = 0;
	if(turn2)
	{
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				if((*chessboard)[i][j] == W_KING)
				{
					found = 1;
					break;
				}
			}
			if(found)
			{
				break;
			}
		}
	}
	else
	{
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				if((*chessboard)[i][j] == B_KING)
				{
					found = 1;
					break;
				}
			}
			if(found)
			{
				break;
			}
		}
	}
	found = 0;
	if(inCheck(j, i, turn2, *chessboard, *hasMoved, *enPassant))
	{
		/* We'll use found as a flag to avoid
		 * making an extra variable.
		 */
		found = 1;
	}

	unmakeMove(y2, x2, y, x, turn2, temp, pieceHolder, &bCastled, &wCastled, hasMoved, chessboard, enPassant);

	/* If the move put the king in check, set
	 * temp to 0 to indicate that it wasn't a
	 * legal move.
	 */
	if(found == 1)
	{
		temp = 0;
	}
	
	/* Return temp whether valid or not! */
	return temp;
}

void doMove(int captured, int* illegalMove, int* turn, int* wturns, int* bturns, int* check, int* mate, int* wclock, int* bclock, int* sttime, int* wCastled, int* bCastled, int (*wCap)[15], int (*bCap)[15], int (*hasMoved)[2][8], int (*chessboard)[8][8], int (*enPassant)[2][8])
{
	/* Loop variables. */
	int i, j;

	/* Used for breaking loops early and such. */
	int temp;

	/* Used to determine if we have a checkmate or
	 * stalemate situation.
	 */
	struct move* validMoves;

	/* If it's the white player's turn, add anything
	 * he captured to his captured pieces box.  Then,
	 * reset black's enPassant variables.  Finally,
	 * increase white's turn counter.
	 *
	 * For black, do the same, but opposite,  tasks.
	 */
	if(*turn)
	{
		for(i=0; i<15; i++)
		{
			if((*wCap)[i] == 0)
			{
				(*wCap)[i] = captured;
				break;
			}
		}
		for(i=0; i<8; i++)
		{
			(*enPassant)[1][i] = 0;
		}
		*wturns += 1;
	}
	else
	{
		for(i=0; i<15; i++)
		{
			if((*bCap)[i] == 0)
			{
				(*bCap)[i] = captured;
				break;
			}
		}
		for(i=0; i<8; i++)
		{
			(*enPassant)[0][i] = 0;
		}
		*bturns += 1;
	}

	/* When a pawn reaches the end of the board, we must make it
	 * a queen.  This changes the values on the chessboard.  On
	 * the next loop through drawGLScene(), the pieces will be
	 * updated visually.  Unless we're on a terribly slow machine,
	 * drawGLScene gets called often enough that the delay isn't
	 * noticable.
	 */
	for(i=0; i<8; i++)
	{
		if((*chessboard)[0][i] == B_PAWN)
		{
			(*chessboard)[0][i] = B_QUEEN;
		}

		if((*chessboard)[7][i] == W_PAWN)
		{
			(*chessboard)[7][i] = W_QUEEN;
		}
	}

	*check = 0;
	temp = 0;
	/* If it's black's turn, we need to find the
	 * white king and determine if it's in check.
	 * This check is for printing the "in check"
	 * warning to the screen.
	 */
	if(!(*turn))
	{
		/* Search chessboard for the white
		 * king, then pass his position to
		 * inCheck(), pretending that it's
		 * the white player's turn.
		 */
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				if((*chessboard)[i][j] == W_KING)
				{
					temp = 1;
					break;
				}
			}
			if(temp)
			{
				break;
			}
		}
		if(inCheck(j, i, !(*turn), *chessboard, *hasMoved, *enPassant))
		{
			*check = 1;
		}
		else
		{
			*check = 0;
		}
	}
	/* It's white's turn, so we need to determine
	 * if the black king is in check.
	 */
	else
	{
		/* Search chessboard for the black
		 * king, then pass his position to
		 * inCheck(), pretending that it's the
		 * black player's turn.
		 */
		for(i=0; i<8; i++)
		{
			for(j=0; j<8; j++)
			{
				if((*chessboard)[i][j] == B_KING)
				{
					temp = 1;
					break;
				}
			}
			if(temp)
			{
				break;
			}
		}
		if(inCheck(j, i, !(*turn), *chessboard, *hasMoved, *enPassant))
		{
			*check = 1;
		}
		else
		{
			*check = 0;
		}
	}

	/* Finally, determine if the next player has any
	 * valid moves at all.  If not, the game ends.
	 * If his king is also in check, checkmate.  Else,
	 * we've reached a stalemate.
	 */
	validMoves = getLegalMoves(!(*turn), *bCastled, *wCastled, chessboard, hasMoved, enPassant);
	if(validMoves == NULL)
	{
		if(*check)
		{
			*mate = 1;
		}
		else
		{
			*mate = 2;
		}
		*illegalMove = 0;
		*check = 0;
	}
	while(validMoves != NULL)
	{
		pop(&validMoves);
	}

	if(*turn)
	{
		*wclock = time(NULL) - *sttime - *bclock;
	}
	else
	{
		*bclock = time(NULL) - *sttime - *wclock;
	}

	*turn = !(*turn);

	return;
}
