#include "client.h"
#include "chesslib.h"

int window;
GLuint base;
GLuint base2;

/* Store the player's color. */
int color;
int aiMode;
char whiteName[32];
char blackName[32];
int sock;
int turn;
struct move* nextMove;

int chessboard[8][8];
int hasMoved[2][8];
int enPassant[2][8];
int wCap[15];
int bCap[15];
int bCastled;
int wCastled;
int check;
int mate;
int wturns;
int bturns;
int wtime;
int btime;
int sttime;
int illegalMove;

int xcrd;
int ycrd;
int xcrd2;
int ycrd2;
int selected;
int input;

void buildFont(void)
{
	/* Store our X display. */
	Display* dpy;

	/* Store information about our X font. */
	XFontStruct* fontInfo;

	/* Initialize each font to hold 96 characters.  Be
	 * sure to do this once for each font!
	 */
	base = glGenLists(96);
	base2 = glGenLists(96);

	/* Connect to our X display. */
	dpy = XOpenDisplay(NULL);
	
	/* Load an X font.  We'll try something fancy, then default
	 * to a system font.  If neither can be found, X fonts are
	 * not supported on this machine.
	 */
	fontInfo = XLoadQueryFont(dpy, "-adobe-courier-bold-r-normal-*-25-*-*-*-*-*-iso8859-1");
	if(fontInfo == NULL)
	{
		fontInfo = XLoadQueryFont(dpy, "fixed");
		if(fontInfo == NULL)
		{
			fprintf(stderr, "No X font available.\n");
		}
	}

	/* Store our X font information in base so we can use it
	 * later and free the font.
	 */
	glXUseXFont(fontInfo->fid, 32, 96, base);
	XFreeFont(dpy, fontInfo);

	/* Repeat the above steps for each font. */
	fontInfo = XLoadQueryFont(dpy, "-adobe-helvetica-bold-r-*-*-34-*-*-*-*-*-iso8859-1");
	if(fontInfo == NULL)
	{
		fontInfo = XLoadQueryFont(dpy, "fixed");
		if(fontInfo == NULL)
		{
			fprintf(stderr, "No X font available.\n");
		}
	}

	glXUseXFont(fontInfo->fid, 32, 96, base2);
	XFreeFont(dpy, fontInfo);

	XCloseDisplay(dpy);
}

void killFont(void)
{
	glDeleteLists(base, 96);
	glDeleteLists(base2, 96);
}

void glPrint(const char* text, int font, ...)
{
	/* Stores the final string to be printed.  Can
	 * be larger to support larger strings.
	 */
	char fmt[256];

	/* Store any additional arguments passed to
	 * glPrint().  This list has a variable length.
	 */
	va_list ap;

	/* If there is no text to print, just return. */
	if(text == NULL)
	{
		return;
	}

	/* Put all variables after "font" into ap. */
	va_start(ap, font);

	/* Format text and ap in a printf() manner and
	 * store the result in fmt.
	 */
	vsprintf(fmt, text, ap);
	va_end(ap);

	/* Print to the screen, using a switch to determine our font.
	 * Be sure to add a new case here for each new font. */
	glPushAttrib(GL_LIST_BIT);
	switch(font) {
	case 0:
		glListBase(base - 32);
	break;
	case 1:
		glListBase(base2 - 32);
	break;
	default:
		glListBase(base - 32);
	break;
	}
	glCallLists(strlen(fmt), GL_UNSIGNED_BYTE, fmt);
	glPopAttrib();
}

void resizeGLScene(int width, int height)
{
	/* Dividing by zero is a terrible idea. */
	if(height==0)
	{
		height=1;
	}

	/* Reset. */
	glViewport(0, 0, width, height);

	/* Calculate the new aspect ratio. */
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45.0f, (GLfloat)width/(GLfloat)height, 0.1f, 100.0f);
	glMatrixMode(GL_MODELVIEW);

}

int initGL(int width, int height)
{
	/* Set background color to black. */
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

	/* Enable clearig of the depth buffer. */
	glClearDepth(1.0f);

	/* Specify the depth test type and enable depth testing.  */
	glDepthFunc(GL_LEQUAL);
	glEnable(GL_DEPTH_TEST);

	/* Enable smooth color shading. */
	glShadeModel(GL_SMOOTH);

	/* This makes our perspective calculations better. */
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	
	/* Set up our initial perspective. */
	resizeGLScene(width, height);

	/* Build our fonts for later use. */
	buildFont();

	/* We return 0, for success. */
	return 0;
}

void drawGLScene(void)
{
	/* Loop variables. */
	int i, j, black;

	/* Clear the screen and depth buffer, then reset the window. */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();

	/* Give us some more space on the screen. */
	glTranslatef(0.0f, 0.0f, -20.0f);

	/* Draw the chess board, 8x8. */
	black = 0;
	glBegin(GL_QUADS);
	for(i=-4; i<4; i++)
	{
		for(j=-4; j<4; j++)
		{
			black ? glColor3f(0.0f, 0.0f, 0.0f) : glColor3f(1.0f, 1.0f, 1.0f);
			glVertex3f((float)(-j-1), (float)i, 0.0f);
			glVertex3f((float)(-j-1), (float)(i+1), 0.0f);
			glVertex3f((float)-j, (float)(i+1), 0.0f);
			glVertex3f((float)-j, (float)i, 0.0f);

			black = !black;
		}
		black = !black;
	}
	glEnd();

	/* Print coordinates for the board.  The first loop prints
	 * 1-8 on the left side of the board; the second loop
	 * prints a-h on the bottom side.
	 */
	glColor3f(1.0f, 1.0f, 1.0f);
	for(i=-4; i<4; i++)
	{
		glRasterPos2f(-4.5f, i+0.2f);
		glPrint("%i", 0, i+5);
	}
	
	for(i=-4; i<4; i++)
	{
		glRasterPos2f(i+0.2f, -4.6f);
		glPrint("%c", 0, i+101);
	}

	/* Add some lines to make the UI nice. */
	glBegin(GL_LINES);
	glColor3f(1.0f, 1.0f, 1.0f);

	/* These four lines outline the chess board. */
	glVertex3f(-4.0f, 4.0f, 0.0f);
	glVertex3f(4.0f, 4.0f, 0.0f);

	glVertex3f(4.0f, 4.0f, 0.0f);
	glVertex3f(4.0f, -4.0f, 0.0f);

	glVertex3f(4.0f, -4.0f, 0.0f);
	glVertex3f(-4.0f, -4.0f, 0.0f);

	glVertex3f(-4.0f, -4.0f, 0.0f);
	glVertex3f(-4.0f, 4.0f, 0.0f);

	/* These four lines outline the pieces captured box on the
	 * white side.
	 */
	glColor3f(0.0f, 0.0f, 1.0f);
	glVertex3f(-5.0f, -2.0f, 0.0f);
	glVertex3f(-9.0f, -2.0f, 0.0f);

	glVertex3f(-9.0f, -2.0f, 0.0f);
	glVertex3f(-9.0f, 2.0f, 0.0f);

	glVertex3f(-9.0f, 2.0f, 0.0f);
	glVertex3f(-5.0f, 2.0f, 0.0f);

	glVertex3f(-5.0f, 2.0f, 0.0f);
	glVertex3f(-5.0f, -2.0f, 0.0f);

	/* These four lines outline the pieces captured box on the
	 * black side.
	 */
	glColor3f(1.0f, 0.0f, 0.0f);
	glVertex3f(5.0f, -2.0f, 0.0f);
	glVertex3f(9.0f, -2.0f, 0.0f);

	glVertex3f(9.0f, -2.0f, 0.0f);
	glVertex3f(9.0f, 2.0f, 0.0f);

	glVertex3f(9.0f, 2.0f, 0.0f);
	glVertex3f(5.0f, 2.0f, 0.0f);

	glVertex3f(5.0f, 2.0f, 0.0f);
	glVertex3f(5.0f, -2.0f, 0.0f);
	glEnd();

	/* Update the appropriate clock every time we go through the
	 * drawGLScene() function.  This will ensure that they appear
	 * accurate on the UI, even if they're off by a few milliseconds
	 * internally.  If the game has ended, don't update the clocks.
	 */
	if(!mate)
	{
		if(turn)
		{
			wtime = time(NULL) - sttime - btime;
		}
		else if(!turn)
		{
			btime = time(NULL) - sttime - wtime;
		}
	}

	/* Print labels for the white side.  These labels include the color
	 * of white's pieces, a clock that tracks the time taken by white's
	 * player to make all of his moves, and how many turns the white player
	 * has had.  Finally, include a * after white's color label if it is
	 * the white player's turn.
	 */
	glColor3f(0.0f, 0.0f, 1.0f);
	glRasterPos2f(-9.0f, 2.1f);
	turn ? glPrint("%s*", 0, whiteName) : glPrint("%s", 0, whiteName);
	glRasterPos2f(-9.0f, -2.5f);
	glPrint("%02i:%02i:%02i(%i)", 0, wtime/3600, (wtime/60)%60, wtime%60, wturns);

	/* Print the same labels for the black side. */
	glColor3f(1.0f, 0.0f, 0.0f);
	glRasterPos2f(5.0f, 2.1f);
	turn ? glPrint("%s", 0, blackName) : glPrint("%s*", 0, blackName);
	glRasterPos2f(5.0f, -2.5f);
	glPrint("%02i:%02i:%02i(%i)", 0, btime/3600, (btime/60)%60, btime%60, bturns);

	/* If the white player has captured any pieces, we want to display
	 * them in his captured pieces box.  We do that here by looking
	 * through wCap[] for any non-zero entries and displaying the pieces
	 * that corresponds to those entries (ex. R for B_ROOK).
	 *
	 * Likewise, we print any white pieces that the black player has
	 * captured.
	 */
	glColor3f(1.0f, 0.0f, 0.0f);
	for(i=0; i<15; i++)
	{
		glColor3f(1.0f, 0.0f, 0.0f);
		glRasterPos2f(-8.9f+0.6*(i%5), 1.3f-(i/5)*0.7);
		switch(wCap[i]) {
		case B_ROOK:
			glPrint("R", 1);
		break;
		case B_KNIGHT:
			glPrint("N", 1);
		break;
		case B_BISHOP:
			glPrint("B", 1);
		break;
		case B_QUEEN:
			glPrint("Q", 1);
		break;
		case B_KING:
			glPrint("K", 1);
		break;
		case B_PAWN:
			glPrint("P", 1);
		break;
		default:
			glPrint("", 1);
		break;
		}

		glColor3f(0.0f, 0.0f, 1.0f);
		glRasterPos2f(5.1f+0.6*(i%5), 1.3f-(i/5)*0.7);
		switch(bCap[i]) {
		case W_ROOK:
			glPrint("R", 1);
		break;
		case W_KNIGHT:
			glPrint("N", 1);
		break;
		case W_BISHOP:
			glPrint("B", 1);
		break;
		case W_QUEEN:
			glPrint("Q", 1);
		break;
		case W_KING:
			glPrint("K", 1);
		break;
		case W_PAWN:
			glPrint("P", 1);
		break;
		default:
			glPrint("", 1);
		break;
		}
	}

	/* Print a thick, color box around one square of the chessboard.
	 * This box acts as a cursor, allowing the player to know which
	 * piece he has selected.  Red square for black, blue for white.
	 */
	glLineWidth(5);
	turn ? glColor3f(0.0f, 0.0f, 1.0f) : glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINES);
	glVertex3f(xcrd, ycrd, 0.0f);
	glVertex3f(xcrd, ycrd+1, 0.0f);

	glVertex3f(xcrd, ycrd+1, 0.0f);
	glVertex3f(xcrd+1, ycrd+1, 0.0f);
	
	glVertex3f(xcrd+1, ycrd+1, 0.0f);
	glVertex3f(xcrd+1, ycrd, 0.0f);

	glVertex3f(xcrd+1, ycrd, 0.0f);
	glVertex3f(xcrd, ycrd, 0.0f);
	glEnd();
	

	/* If the player has selected a piece, we draw a bright yellow
	 * box around the selected piece.  He still uses the red/blue
	 * cursor to move around the board and select his piece's
	 * destination.
	 */
	if(selected)
	{
		glLineWidth(5);
		glColor3f(1.0f, 1.0f, 0.0f);
		glBegin(GL_LINES);
		glVertex3f(xcrd2, ycrd2, 0.0f);
		glVertex3f(xcrd2, ycrd2+1, 0.0f);

		glVertex3f(xcrd2, ycrd2+1, 0.0f);
		glVertex3f(xcrd2+1, ycrd2+1, 0.0f);

		glVertex3f(xcrd2+1, ycrd2+1, 0.0f);
		glVertex3f(xcrd2+1, ycrd2, 0.0f);

		glVertex3f(xcrd2+1, ycrd2, 0.0f);
		glVertex3f(xcrd2, ycrd2, 0.0f);
		glEnd();
	}

	/* Set the line width back to 1 for future line printing. */
	glLineWidth(1);

	/* If a player has made an illegal move, we must inform them.
	 * This is accomplished with a colored message below the board.
	 * These colors correspond to the player colors.
	 */
	if(illegalMove)
	{
		turn ? glColor3f(0.0f, 0.0f, 1.0f) : glColor3f(1.0f, 0.0f, 0.0f);
		glRasterPos3f(-2.1f, -5.3f, 0.0f);
		glPrint("Illegal Move!", 0);
	}
	if(wturns < 1 && bturns < 1 && !illegalMove)
	{
		color ? glColor3f(0.0f, 0.0f, 1.0f) : glColor3f(1.0f, 0.0f, 0.0f);
		glRasterPos3f(-2.1f, -5.3f, 0.0f);
		glPrint("You are %s", 0, (color ? "BLUE" : "RED"));
	}

	/* Similarly, if a player is in check, we must inform him.  Here,
	 * we print a message above the board.  Again, the colors match
	 * the player colors.
	 */
	if(check)
	{
		turn ? glColor3f(0.0f, 0.0f, 1.0f) : glColor3f(1.0f, 0.0f, 0.0f);
		turn ? glRasterPos3f(-2.6f, 4.3f, 0.0f) : glRasterPos3f(-2.3f, 4.3f, 0.0f);
		turn ? glPrint("Blue is in CHECK", 0) : glPrint("Red is in CHECK", 0);
	}

	/* If the game is over, we must let the players know! */
	if(mate)
	{
		if(mate == 1)
		{
			!turn ? glColor3f(0.0f, 0.0f, 1.0f) : glColor3f(1.0f, 0.0f, 0.0f);
			!turn ? glRasterPos3f(-3.35f, 4.3f, 0.0f) : glRasterPos3f(-3.175f, 4.3f, 0.0f);
			!turn ? glPrint("Checkmate! BLUE wins!", 0) : glPrint("Checkmate! RED wins!", 0);
		}
		else
		{
			glColor3f(1.0f, 1.0f, 1.0f);
			glRasterPos3f(-1.5f, 4.3f, 0.0f);
			glPrint("Stalemate!", 0);
		}
	}

	/* We use capital letters to represent each chess piece on the
	 * board.  Here, we loop through chessboard to determine any
	 * non-zero entries (pieces) and print their representing
	 * letters to the screen.
	 * 	R : Rook
	 * 	N : Knight
	 * 	B : Bishop
	 * 	Q : Queen
	 * 	K : King
	 * 	P : Pawn
	 */
	for(i=0; i<8; i++)
	{
		for(j=0; j<8; j++)
		{
			switch(chessboard[i][j]) {
			case B_ROOK:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("R", 1);
			continue;
			case B_KNIGHT:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("N", 1);
			continue;
			case B_BISHOP:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("B", 1);
			continue;
			case B_QUEEN:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("Q", 1);
			continue;
			case B_KING:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("K", 1);
			continue;
			case B_PAWN:
				glColor3f(0.75f, 0.0f, 0.0f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("P", 1);
			continue;
			case W_ROOK:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("R", 1);
			continue;
			case W_KNIGHT:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("N", 1);
			continue;
			case W_BISHOP:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("B", 1);
			continue;
			case W_QUEEN:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("Q", 1);
			continue;
			case W_KING:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("K", 1);
			continue;
			case W_PAWN:
				glColor3f(0.0f, 0.0f, 0.75f);
				glRasterPos3f(j-3.8f, i-3.8f, 0.1f);
				glPrint("P", 1);
			continue;
			default:
			continue;
			}
		}
	}

	/* Swap buffers -- we have two! */
	glutSwapBuffers();
}

void parseMove(char* buffer)
{
	/* a1:a1, x-axis is the a, y-axis is the 1 */
	/* a = 97, b = 98, ..., f = 102 */
	nextMove->y2 = (int)(atoi(&buffer[1])-4);
	nextMove->x2 = (int)(buffer[0]-101);
	nextMove->y = (int)(atoi(&buffer[4])-4);
	nextMove->x = (int)(buffer[3]-101);
	nextMove->type = chessboard[(nextMove->y2)+4][(nextMove->x2)+4];
	nextMove->value = -10000000;
	nextMove->next = NULL;
	return;
}

char* unparseMove()
{
	char* temp;
	temp = (char*)calloc(6, sizeof(char));
	if(nextMove->x2 == 'q')
	{
		strncpy(temp, "quit", 4);
	}
	else
	{
		temp[0] = ((int)nextMove->x2 + 101);
		temp[1] = ((int)nextMove->y2 + 52);
		temp[2] = ':';
		temp[3] = ((int)nextMove->x + 101);
		temp[4] = ((int)nextMove->y + 52);
		temp[5] = '\0';
	}
	return temp;
}

void playGame()
{
	int bytes;
	int quitFlag;
	int legalMove;
	int flag;
	int pieceHolder;
	char buffer[6];
	char* moveHolder;
	quitFlag = 0;
	legalMove = 0;

	if(color)
	{
		if(turn)
		{
			if(!input)
			{
				drawGLScene();
				return;
			}
			input = 0;
			bzero(buffer, 6);
			moveHolder = unparseMove();
			strcpy(buffer, moveHolder);
			free(moveHolder);
			if((bytes = send(sock, buffer, 6, 0)) > 0)
			{
				fprintf(stdout, "Sent %s to server.\n", buffer);
			}
			bzero(buffer, 6);
			if((bytes = recv(sock, buffer, 6, 0)) == 6)
			{
				if(strncmp(buffer, "x", 1) == 0)
				{
					fprintf(stdout, "Tried to make an illegal move.\n");
					illegalMove = 1;
				}
				else
				{
					fprintf(stdout, "Received permission to move %s.\n", buffer);
					legalMove = 1;
				}
			}
			else
			{
				fprintf(stdout, "Failed to receive a response from the server.\n");
			}
			if(legalMove)
			{
				illegalMove = 0;
				flag = 0;
				pieceHolder = 0;
				flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
				pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
				doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
			}
			legalMove = 0;
		}
		else
		{
			alarm(1);
			bzero(buffer, 6);
			if((bytes = recv(sock, buffer, 6, 0)) > 0)
			{
				if(strncmp(buffer, "quit", 4) == 0)
				{
					quitFlag = 1;
				}
			}
			if(quitFlag == 1)
			{
				killFont();
				free(nextMove);
				glutDestroyWindow(window);
				exit(1);							
			}
			fprintf(stdout, "%s has moved %s.\n", blackName, buffer);
			parseMove(buffer);
			flag = 0;
			pieceHolder = 0;
			flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
			pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
			doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
			alarm(0);
		}
	}
	else
	{
		if(!turn)
		{
			if(!input)
			{
				drawGLScene();
				return;
			}
			input = 0;
			bzero(buffer, 6);
			moveHolder = unparseMove();
			strcpy(buffer, moveHolder);
			free(moveHolder);
			if((bytes = send(sock, buffer, 6, 0)) > 0)
			{
				fprintf(stdout, "Sent %s to server.\n", buffer);
			}
			bzero(buffer, 6);
			if((bytes = recv(sock, buffer, 6, 0)) == 6)
			{
				if(strncmp(buffer, "x", 1) == 0)
				{
					fprintf(stdout, "Tried to make an illegal move.\n");
					illegalMove = 1;
				}
				else
				{
					fprintf(stdout, "Received permission to move %s.\n", buffer);
					legalMove = 1;
				}
			}
			else
			{
				fprintf(stdout, "Failed to receive a response from the server.\n");
			}
			if(legalMove)
			{
				illegalMove = 0;
				flag = 0;
				pieceHolder = 0;
				flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
				pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
				doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
			}
			legalMove = 0;
		}
		else
		{
			alarm(1);
			bzero(buffer, 6);
			if((bytes = recv(sock, buffer, 6, 0)) > 0)
			{
				if(strncmp(buffer, "quit", 4) == 0)
				{
					quitFlag = 1;
				}
			}
			if(quitFlag == 1)
			{
				killFont();
				free(nextMove);
				glutDestroyWindow(window);
				exit(1);							
			}
			fprintf(stdout, "%s has moved %s.\n", whiteName, buffer);
			parseMove(buffer);
			flag = 0;
			pieceHolder = 0;
			flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
			pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
			doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
			alarm(0);
		}
	}

	return;
}

void keyPressed(unsigned char key, int x, int y)
{
	#ifdef DKEYS
	/* Loop variables. */
	int i, j;
	#endif /* DKEYS */

	/* Temporary variables. */
	int temp2;

	#ifdef DKEYS
	/* Store a list of legal moves.  This
	 * is for debugging.
	 */
	struct move* tempMoves;
	tempMoves = NULL;
	#endif /* DKEYS */

	/* We sleep here to avoid thrashing. */
	usleep(100);

	/* Exit on ESCAPE. */
	if(key == ESCAPE)
	{
		temp2 = 0;
		if((temp2 = send(sock, "quit\0\0", 6, 0)) > 0)
		{
			fprintf(stdout, "Told server to quit.\n");
		}
		else
		{
			fprintf(stderr, "Failed to tell server we're quitting.\n");
			fprintf(stderr, "Exiting anyway...\n");
		}
		killFont();
		free(nextMove);
		glutDestroyWindow(window);
		exit(1);
	}

	/* Select a piece or a destination. */
	if(key == SPACE_BAR && ((turn && color == 1) || (!turn && color == 0)))
	{
		/* If the game is over or if the current player is
		 * controlled by AI, don't process input. */
		if(mate)
		{
			return;
		}

		/* If the user has already selected a piece, we follow
		 * the logic below.  Otherwise, we simply select the piece
		 * that's highlighted.
		 */
		if(selected)
		{
			input = 1;
			nextMove->y2 = ycrd2;
			nextMove->x2 = xcrd2;
			nextMove->y = ycrd;
			nextMove->x = xcrd;
			nextMove->type = chessboard[ycrd2+4][xcrd2+4];
		}
		/* If a piece hasn't been selected, set xcrd2 and ycrd2 to
		 * the cursor's current position.  These two variables store
		 * the location of the selected piece.
		 */
		else
		{
			xcrd2 = xcrd;
			ycrd2 = ycrd;
		}

		/* If we haven't selected a piece yet, now we have.  If we
		 * already had, now we haven't.
		 */
		selected = !selected;

		/* In all cases, return because we're finished. */
		return;
	}

	/* Move the cursor left. */
	if(key == LEFT)
	{
		/* If the game is over, don't process game input. */
		if(mate)
		{
			return;
		}

		if(xcrd != -4)
		{
			xcrd--;
		}
		return;
	}

	/* Move the cursor right. */
	if(key == RIGHT)
	{
		/* If the game is over, don't process game input. */
		if(mate)
		{
			return;
		}

		if(xcrd != 3)
		{
			xcrd++;
		}
		return;
	}

	/* Move the cursor up. */
	if(key == UP)
	{
		/* If the game is over, don't process game input. */
		if(mate)
		{
			return;
		}

		if(ycrd != 3)
		{
			ycrd++;
		}
		return;
	}

	/* Move the cursor down. */
	if(key == DOWN)
	{
		/* If the game is over, don't process game input. */
		if(mate)
		{
			return;
		}

		if(ycrd != -4)
		{
			ycrd--;
		}
		return;
	}

	#ifdef DKEYS
	/* Print the enPassant array. */
	if(key == EP_DEBUG)
	{
		for(i=1; i>=0; i--)
		{
			for(j=0; j<8; j++)
			{
				fprintf(stderr, "%i ", enPassant[i][j]);
			}
			fprintf(stderr, "\n");
		}
		fprintf(stderr, "\n");
		return;
	}

	/* Print the hasMoved array. */
	if(key == HM_DEBUG)
	{
		for(i=1; i>=0; i--)
		{
			for(j=0; j<8; j++)
			{
				fprintf(stderr, "%i ", hasMoved[i][j]);
			}
			fprintf(stderr, "\n");
		}
		fprintf(stderr, "\n");
		return;
	}

	/* Print the chessboard array. */
	if(key == B_DEBUG)
	{
		for(i=7; i>=0; i--)
		{
			for(j=0; j<8; j++)
			{
				fprintf(stderr, "%i ", chessboard[i][j]);
			}
			fprintf(stderr, "\n");
		}
		fprintf(stderr, "\n");
		return;
	}
	/* Prints the bCap and wCap arrays. */
	if(key == CAP_DEBUG)
	{
		fprintf(stderr, "bCap: ");
		for(i=0; i<15; i++)
		{
			fprintf(stderr, "%i ", bCap[i]);
		}
		fprintf(stderr, "\n");
		fprintf(stderr, "wCap: ");
		for(i=0; i<15; i++)
		{
			fprintf(stderr, "%i ", wCap[i]);
		}
		fprintf(stderr, "\n");
		return;
	}
	if(key == GET_MOVES)
	{
		tempMoves = getLegalMoves(turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
		while(tempMoves != NULL)
		{
			fprintf(stderr, "%i,%i to %i,%i\n", tempMoves->y2+4, tempMoves->x2+4, tempMoves->y+4, tempMoves->x+4);
			pop(&tempMoves);
		}
		fprintf(stderr, "\n");
		return;
	}
	#endif /* DKEYS */
}

void initChessboard(void)
{
	/* Loop variables. */
	int i, j;

	/* Place the pieces in their proper starting locations. */
	chessboard[7][0] = B_ROOK;
	chessboard[7][1] = B_KNIGHT;
	chessboard[7][2] = B_BISHOP;
	chessboard[7][3] = B_QUEEN;
	chessboard[7][4] = B_KING;
	chessboard[7][5] = B_BISHOP;
	chessboard[7][6] = B_KNIGHT;
	chessboard[7][7] = B_ROOK;

	chessboard[0][0] = W_ROOK;
	chessboard[0][1] = W_KNIGHT;
	chessboard[0][2] = W_BISHOP;
	chessboard[0][3] = W_QUEEN;
	chessboard[0][4] = W_KING;
	chessboard[0][5] = W_BISHOP;
	chessboard[0][6] = W_KNIGHT;
	chessboard[0][7] = W_ROOK;

	for(i=0; i<8; i++)
	{
		chessboard[6][i] = B_PAWN;
		chessboard[1][i] = W_PAWN;
	}

	/* Set the rest of the squares to zero. */
	for(i=2; i<6; i++)
	{
		for(j=0; j<8; j++)
		{
			chessboard[i][j] = 0;
		}
	}

	/* White starts, so set turn. */
	turn = 1;

	/* Set the coordinates of the cursors to the a1 square. */
	xcrd = -4;
	ycrd = -4;
	xcrd2 = -4;
	ycrd2 = -4;

	/* Set the turn counter for each player to 0. */
	wturns = 0;
	bturns = 0;

	/* Set the clock for each player to 0 seconds. */
	wtime = 0;
	btime = 0;

	/* Get the time at the start of the game. */
	sttime = time(NULL);

	/* Set legal move flags to 0. */
	illegalMove = 0;
	check = 0;
	mate = 0;

	/* Set selected to 0, too. */
	selected = 0;

	/* Set the hasMoved array to 0 for all pieces. */
	for(i=0; i<2; i++)
	{
		for(j=0; j<8; j++)
		{
			hasMoved[i][j] = 0;
		}
	}

	/* Clear the captured pieces arrays. */
	for(i=0; i<15; i++)
	{
		bCap[i] = 0;
		wCap[i] = 0;
	}

	/* Clear any enPassant flags. */
	for(i=0; i<2; i++)
	{
		for(j=0; j<8; j++)
		{
			enPassant[i][j] = 0;
		}
	}

	bCastled = 0;
	wCastled = 0;

	input = 0;

	nextMove->y2 = 0;
	nextMove->x2 = 0;
	nextMove->y = 0;
	nextMove->x = 0;
	nextMove->type = 0;
	nextMove->value = -10000000;
	nextMove->next = NULL;
}

int main(int argc, char* argv[])
{
	/* Temporary variable. */
	int i;

	/* What port to listen on */
	int port;

	/* Server to connect to */
	char* server;

	/* Return value from socketStuff() */
	int ret;

	/* Temporary storage for player's name. */
	char name[32];

	/* Setup defaults */
	port = 11111;
	server = NULL;
	aiMode = 0;
	strcpy(name, "Lazy Bastard!");
	bzero(whiteName, 32);
	bzero(blackName, 32);

	/* Set up alarm signal handler. */
	signal(SIGALRM, sigAlarmHandler);

	/* Parse command line args */
	while((i = getopt(argc, argv, "hs:p:n:a:")) != EOF)
	{
		switch(i)
		{
			case 'p':
				port = atoi(optarg);
				break;
			case 's':
				server = optarg;
				break;
			case 'n':
				if(strlen(optarg) > 32)
				{
					fprintf(stderr, "Name must be 32 characters or less.\n");
					return BADARG;
				}
				bzero(name, sizeof(name));
				strcpy(name, optarg);
				break;
			case 'a':
				if(aiMode >= 0 && aiMode <= 3)
				{
					aiMode = atoi(optarg);
				}
				else
				{
					fprintf(stderr, "AI Mode must be 0, 1, 2, or 3.\n");
					return BADARG;
				}
				break;
			case '?':
			case 'h':
				usage();
				return BADARG;
		}
	}

	/* Make sure they gave us a server */
	if(!server)
	{
		usage();
		return BADARG;
	}

	ret = socketStuff(server, port, name);

	if(ret == SUCCESS)
	{
		/* Initialize GLUT and pass any command line arguments. */
		glutInit(&argc, argv);

		/* Set up the display mode. */
		glutInitDisplayMode(GLUT_RGBA | GLUT_DOUBLE | GLUT_ALPHA | GLUT_DEPTH);

		/* Set the window's initial size. */
		glutInitWindowSize(1024, 768);

		/* Position the window in the top-left corner of the screen. */
		glutInitWindowPosition(0, 0);

		/* Open the window. */
		window = glutCreateWindow("Chess");

		/* Register our drawing function with OpenGL. */
		glutDisplayFunc(&drawGLScene);

		/* Register our window resizing function with OpenGL. */
		glutReshapeFunc(&resizeGLScene);

		/* Register our key sniffing fuction with OpenGL. */
		glutKeyboardFunc(&keyPressed);

		/* Idle in the playGame() function. */
		glutIdleFunc(&playGame);

		/* Initialize the window. */
		if(initGL(1024, 768))
		{
			fprintf(stderr, "Failed to initialize GL window.  Exiting.\n");
			return 1;
		}

		/* Allocate memory for nextMove. */
		nextMove = (struct move*)malloc(sizeof(struct move));

		/* Initialize the chessboard and any gamestate variables. */
		initChessboard();

		/* Start the event processing engine.  This should start our
		 * playGame() fuction and process any keyboard input.
		 */
		glutMainLoop();

		/* Close our socket because we're exiting. */
		close(sock);
	}

	return ret;
}

void usage()
{
	fprintf(stderr, "\nchess-client - Connects to a remote server to play chess.\n");
	fprintf(stderr, "usage: chess-client -s server [-a mode] [-p port] [-n name]\n");
	fprintf(stderr, "	Server is the server to connect to.\n");
	fprintf(stderr, "	Port is the port to connection to on the server (default: 11111).\n");
	fprintf(stderr, "	Mode is the AI mode.  0 for no AI, 1-3 for AI (novice-master).\n");
	fprintf(stderr, "	Name is the name you wish to go by.\n");
}

int socketStuff(const char* server, const int port, const char* name)
{
	/* Number of bytes read in */
	int in;

	/* The buffer for storing the outgoing command and recieved data */
	char buf[MAXCMD]; 

	/* Sockaddr for holding server information */
	struct sockaddr_in serv_addr;

	/* Address of remote server */
	struct hostent *hp;

	/* Get server's remote address */
	bzero((void*)&serv_addr, sizeof(serv_addr));
	if((hp = gethostbyname(server)) == NULL)
	{
		fprintf(stderr, "Cannot convert hostname.\n");
		return HOSTERR;
	}
	bcopy(hp->h_addr, (char*)&serv_addr.sin_addr, hp->h_length);
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);

	/* Open the socket */
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr, "Could not open a socket on port %i.\n", port);
		return SOCKERR;
	}

	/* Connect to the server */
	if((connect(sock, (struct sockaddr*)&serv_addr, sizeof(serv_addr)) < 0))
	{
		fprintf(stderr, "Could not connect to remote host %s.\n", server);
		return REMOTEHOSTERR;
	}

	/* Send our command */
	if(send(sock, name, strlen(name), 0) < 0)
	{
		fprintf(stderr, "Failed to send name to server.\n");
	}
	else /* Only go on if the command got sent */
	{
		bzero(buf, MAXCMD);
		/* Get the server's response */
		if((in = recv(sock, buf, 1, 0)) > 0)
		{
			color = (atoi(buf)-1);
			fprintf(stdout, "Got a response from the server.  Player is %i.\n", color);
			if(color == 1)
			{
				strcpy(whiteName, name);
				bzero(buf, strlen(buf));
				switch(aiMode) {
				case 0:
					memcpy(buf, "0", 1);
				break;
				case 1:
					memcpy(buf, "1", 1);
				break;
				case 2:
					memcpy(buf, "2", 1);
				break;
				case 3:
					memcpy(buf, "3", 1);
				break;
				default:
					memcpy(buf, "0", 1);
				break;
				}
				if(send(sock, buf, 1, 0) > 0)
				{
					fprintf(stdout, "AI Mode set to %i.\n", atoi(buf));
				}
				else
				{
					fprintf(stderr, "Failed to set AI mode.\n");
					close(sock);
					return AIERR;
				}
				bzero(buf, strlen(buf));
				if((in = recv(sock, buf, 32, 0)) > 0)
				{
					fprintf(stdout, "The black player is named %s.\n", buf);
					strcpy(blackName, buf);
				}
				else
				{
					fprintf(stderr, "Failed to get black player's name.\n");
					close(sock);
					return NAMEERR;
				}
			}
			else
			{
				strcpy(blackName, name);
				bzero(buf, MAXCMD);
				if((in = recv(sock, buf, 32, 0)) > 0)
				{
					fprintf(stdout, "The white player is named %s.\n", buf);
					strcpy(whiteName, buf);
				}
				else
				{
					fprintf(stderr, "Failed to get white player's name.\n");
					close(sock);
					return NAMEERR;
				}
			}
		}
		else
		{
			fprintf(stdout, "Failed to get server response.  Exiting.\n");
			close(sock);
			return RECVERR;
		}
	}
	
	return SUCCESS;
}

void sigAlarmHandler(int signum)
{
	if(signum == SIGALRM)
	{
		signal(SIGALRM, sigAlarmHandler);
		alarm(1);
		drawGLScene();
	}
}
