#include "server.h"
#include "chesslib.h"

/* Socket descriptors for our two players. */
int listensock;
int sendsock;
int sendsock2;

/* Store the names of our two players. */
char* whiteName;
char* blackName;

/* Keep track of the AI mode that white selected. */
int aiMode;

/* We need to always know whose turn it is. */
int turn;

/* We also need to keep track of the chessboard and any
 * other important game state variables.
 */
int chessboard[8][8];
int hasMoved[2][8];
int enPassant[2][8];
int bCap[15];
int wCap[15];
int bCastled;
int wCastled;
int check;
int mate;
int illegalMove;

/* AI variables. */
int bAIDiff;
int bAggressive;

/* For clocks. */
int btime;
int wtime;
int sttime;

/* Keep track of turns. */
int bturns;
int wturns;

/* We can use a struct to make moving pieces easier. */
struct move* nextMove;

void parseMove(char* buffer)
{
	/* a1:a1, x-axis is the a, y-axis is the 1 */
	/* a = 97, b = 98, ..., f = 102 */
	nextMove->y2 = (int)(atoi(&buffer[1])-4);
	nextMove->x2 = (int)(buffer[0]-101);
	nextMove->y = (int)(atoi(&buffer[4])-4);
	nextMove->x = (int)(buffer[3]-101);
	nextMove->type = chessboard[(nextMove->y2)+4][(nextMove->x2)+4];
	nextMove->value = -10000000;
	nextMove->next = NULL;
	return;
}

char* unparseMove()
{
	char* temp;
	temp = (char*)calloc(6, sizeof(char));
	if(nextMove->x2 == 'q')
	{
		strncpy(temp, "quit", 4);
	}
	else
	{
		temp[0] = ((int)nextMove->x2 + 101);
		temp[1] = ((int)nextMove->y2 + 52); 
		temp[2] = ':';
		temp[3] = ((int)nextMove->x + 101);
		temp[4] = ((int)nextMove->y + 52);
		temp[5] = '\0';
	}
	return temp;
}

void playGame()
{
	char buffer[6];
	int quitFlag;
	int pieceHolder;
	int flag;
	int legalMove = 0;
	int bytes;
	quitFlag = 0;
	bytes = 0;

	while(quitFlag == 0)
	{
		if(turn)
		{
			fprintf(stdout, "It is %s's turn.\n", whiteName);
			legalMove = 0;
			while(legalMove == 0 && quitFlag == 0)
			{
				bzero(buffer, 6);
				while((bytes = recv(sendsock, buffer, 6, 0)) > 0)
				{
					if(strncmp(buffer, "quit", 4) == 0)
					{
						fprintf(stdout, "%s has quit the game.  Letting %s know.\n", whiteName, blackName);
						send(sendsock2, "quit\0\0", 6, 0);
						quitFlag = 1;
					}

					break;
				}
				if(quitFlag == 1)
				{
					continue;
				}
				parseMove(buffer);
				flag = 0;
				pieceHolder = 0;
				if((flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant)) > 0)
				{
					pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
					doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
					fprintf(stdout, "%s has moved %s.\n", whiteName, buffer);
					if((bytes = send(sendsock, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", whiteName);
					}
					if((bytes = send(sendsock2, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", blackName);
					}
					legalMove = 1;
					continue;
				}
				else
				{
					if((bytes = send(sendsock, "x\0\0\0\0\0", 6, 0)) > 0)
					{
						fprintf(stdout, "Denied %s's move attempt (%s).\n", whiteName, buffer);
					}
				}
			}
			if(quitFlag == 1)
			{
				continue;
			}
		}
		else
		{
			fprintf(stdout, "It is %s's turn.\n", blackName);
			legalMove = 0;
			while(legalMove == 0 && quitFlag == 0)
			{
				bzero(buffer, 6);
				while((bytes = recv(sendsock2, buffer, 6, 0)) > 0)
				{
					if(strncmp(buffer, "quit", 4) == 0)
					{
						fprintf(stdout, "%s has quit the game.  Letting %s know.\n", blackName, whiteName);
						send(sendsock, "quit\0\0", 6, 0);
						quitFlag = 1;
					}

					break;
				}
				if(quitFlag == 1)
				{
					continue;
				}
				parseMove(buffer);
				flag = 0;
				pieceHolder = 0;
				if((flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant)) > 0)
				{
					pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
					doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
					fprintf(stdout, "%s has moved %s.\n", blackName, buffer);
					if((bytes = send(sendsock, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", whiteName);
					}
					if((bytes = send(sendsock2, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", blackName);
					}
					legalMove = 1;
					continue;
				}
				else
				{
					if((bytes = send(sendsock2, "x\0\0\0\0\0", 6, 0)) > 0)
					{
						fprintf(stdout, "Denied %s's move attempt (%s).\n", blackName, buffer);
					}
				}
			}
			if(quitFlag == 1)
			{
				continue;
			}
		}
		if(quitFlag == 1)
		{
			continue;
		}
	}
	return;
}

void playAIGame()
{
	char buffer[6];
	int quitFlag;
	int pieceHolder;
	int flag;
	int legalMove = 0;
	int bytes;
	quitFlag = 0;
	bytes = 0;

	while(quitFlag == 0)
	{
		if(turn)
		{
			fprintf(stdout, "It is %s's turn.\n", whiteName);
			legalMove = 0;
			while(legalMove == 0 && quitFlag == 0)
			{
				bzero(buffer, 6);
				while((bytes = recv(sendsock, buffer, 6, 0)) > 0)
				{
					if(strncmp(buffer, "quit", 4) == 0)
					{
						fprintf(stdout, "%s has quit the game.\n", whiteName);
						quitFlag = 1;
					}

					break;
				}
				if(quitFlag == 1)
				{
					continue;
				}
				parseMove(buffer);
				flag = 0;
				pieceHolder = 0;
				if((flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant)) > 0)
				{
					pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
					doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
					fprintf(stdout, "%s has moved %s.\n", whiteName, buffer);
					if((bytes = send(sendsock, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", whiteName);
					}
					legalMove = 1;
					continue;
				}
				else
				{
					if((bytes = send(sendsock, "x\0\0\0\0\0", 6, 0)) > 0)
					{
						fprintf(stdout, "Denied %s's move attempt (%s).\n", whiteName, buffer);
					}
				}
			}
			if(quitFlag == 1)
			{
				continue;
			}
		}
		else
		{
			nextMove->type = 0;
			nextMove->y2 = 0;
			nextMove->x2 = 0;
			nextMove->y = 0;
			nextMove->x = 0;
			nextMove->value = -10000000;
			fprintf(stdout, "It is %s's turn.\n", blackName);
			legalMove = 0;
			while(legalMove == 0 && quitFlag == 0)
			{
				negaMax(PLIES, -10000000, 10000000, turn);
				if((flag = isValidMove(nextMove->type, nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, bCastled, wCastled, &chessboard, &hasMoved, &enPassant)) > 0)
				{
					pieceHolder = makeMove(nextMove->y2, nextMove->x2, nextMove->y, nextMove->x, turn, flag, &bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
					doMove(pieceHolder, &illegalMove, &turn, &wturns, &bturns, &check, &mate, &wtime, &btime, &sttime, &wCastled, &bCastled, &wCap, &bCap, &hasMoved, &chessboard, &enPassant);
					bzero(buffer, 6);
					strcpy(buffer, unparseMove());
					fprintf(stdout, "%s has moved %s.\n", blackName, buffer);
					if((bytes = send(sendsock, buffer, 6, 0)) > 0)
					{
						fprintf(stdout, "Informed %s of the move.\n", whiteName);
					}
					legalMove = 1;
					continue;
				}
				else
				{
					fprintf(stderr, "The AI made a crazy move again!\n");
				}
			}
			if(quitFlag == 1)
			{
				continue;
			}
		}
		if(quitFlag == 1)
		{
			continue;
		}
	}

	return;
}

int evaluate(int turn2)
{
	int i, j, k, l;
	int temp;
	int value;
	struct move* tempMoves;

	value = 0;
	temp = 0;
	for(i=0; i<8; i++)
	{
		for(j=0; j<8; j++)
		{
			switch(chessboard[i][j]) {
			case B_ROOK:
				value += -520 + (-bAggressive*260);
				/* Bonus points for any pieces the rook can attack, even X-Ray
				 * style.  If the rook is blocked, this "bonus" is 0.  Else,
				 * it can be up to 20 for a rook attacking 10 squares.
				 *
				 * If any of these attackable squares is an enemy King, Queen,
				 * or Rook, add 8 more bonus points.
				 */
				if(bAIDiff > 1 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(((i == k) || (j == l)) && chessboard[k][l] > B_PAWN)
							{
								temp += 2;
								if(chessboard[k][l] == W_ROOK
									|| chessboard[k][l] == W_QUEEN
									|| chessboard[k][l] == W_KING)
								{
									value -= 8;
								}
							}
						}
					}
					value -= max(18, temp);
				}
				temp = 0;
				/* Rooks get bonus points for being in a file with no friendly pawns.
				 * They also get a bonus if their file contains no enemy pawns.
				 */
				for(k=0; k<8; k++)
				{
					if(chessboard[k][j] == B_PAWN)
					{
						temp = 1;
					}
				}
				if(!temp)
				{
					value -= 10;
				}
				temp = 0;
				for(k=0; k<8; k++)
				{
					if(chessboard[k][j] == W_PAWN)
					{
						temp = 1;
					}
				}
				if(!temp)
				{
					value -= 4;
				}
				temp = 0;
				/* Finally, penalize rooks for being far away from the enemy
				 * king.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == W_KING)
							{
								temp = 1;
							}
							if(temp) break;
						}
						if(temp) break;
					}
					value -= 2*min((i-k), (j-l));
					temp = 0;
				}
			break;
			case B_KNIGHT:
				value += -330 + (-bAggressive * 165);
				/* Bonus for proximity to center of the board.  Basically,
				 * 5 points for each square.  This results in 0 at the
				 * corners and 30 in the four center squares.
				 */
				if(i <= 3 && j <= 3)
				{
					value -= (5*(i-0) + 5*(j-0));
				}
				else if(i <= 3 && j > 3)
				{
					value -= (5*(i-0) + 5*(7-j));
				}
				else if(i > 3 && j > 3)
				{
					value -= (5*(7-i) + 5*(7-j));
				}
				else
				{
					value -= (5*(7-i) + 5*(j-0));
				}

				/* Knights are given a bonus for being within 2 squares
				 * of each enemy piece.  This bonus varies with game
				 * length, as follows:
				 * 	early (<15 turns) : 2
				 * 	middle (<30 turns) : 3
				 * 	late (>30 turns) : 4
				 */
				for(k=0; k<8; k++)
				{
					for(l=0; l<8; l++)
					{
						if(chessboard[k][l] > B_PAWN)
						{
							if(abs(l-j) <=2 || abs(k-i) <= 2)
							{
								value -= (2 + max(2,(bturns%15)));
							}
						}
					}
				}

				/* One point penalty for each square between the knight
				 * and either king.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == W_KING || chessboard[k][l] == B_KING)
							{
								value += (abs(k-i) + abs(l-j));
							}
						}
					}
				}
			break;
			case B_BISHOP:
				value += -330 + (-bAggressive * 165);
				/* Bishops get a bonus as material falls off the board.  Initially,
				 * this bonus is 0.  During late game (>30 moves), this bonus is
				 * 10.
				 */
				if(bturns > 30)
				{
					value -= 10;
				}
				else
				{
					temp = 0;
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] > 0 && chessboard[k][l] > B_PAWN)
							{
								switch(chessboard[k][l]) {
								case W_ROOK:
									temp += 520;
								break;
								case W_KNIGHT:
								case W_BISHOP:
									temp += 330;
								break;
								case W_QUEEN:
									temp += 980;
								break;
								case W_PAWN:
									temp += 100;
								break;
								default:
								break;
								}
							}
						}
					}
					value -= min(0,(8-(temp % 500)));
				}
				temp = 0;

				/* Bonus points for any pieces the bishop can attack, even X-Ray
				 * style.  If the bishop is blocked, this "bonus" is -4.  Else,
				 * it can be up to 18 for a bishop attacking 12 squares.
				 *
				 * If any of these attackable squares is an enemy King, Queen,
				 * or Rook, add 8 more bonus points.
				 */
				if(bAIDiff > 1 || bAIDiff == 0)
				{
					value += 4;
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(abs(i-k) == abs(j-l) && chessboard[k][l] > B_PAWN)
							{
								temp += 2;
								if(chessboard[k][l] == W_ROOK
									|| chessboard[k][l] == W_QUEEN
									|| chessboard[k][l] == W_KING)
								{
									value -= 8;
								}
							}
						}
					}
					value -= max(18, temp);
					temp = 0;
				}

				/* Bishops also get a bonus for being closer to the center
				 * of the board.  On the edge, they get 14 points.  In
				 * the center, they get 22.
				 */
				if(i >= 3 && i <= 4)
				{
					value -= 11;
				}
				else if(i >= 2 && i <= 5)
				{
					value -= 10;
				}
				else if(i >= 1 && i <= 6)
				{
					value -= 9;
				}
				else
				{
					value -= 7;
				}
				if(j >= 3 && j <= 4)
				{
					value -= 11;
				}
				else if(j >= 2 && j <= 5)
				{
					value -= 10;
				}
				else if(j >= 1 && i <= 6)
				{
					value -= 9;
				}
				else
				{
					value -= 7;
				}
			break;
			case B_QUEEN:
				value += -980 + (-bAggressive * 460);
				/* After the opening (5 turns), Queens get a penalty
				 * for their distance from the enemy king.
				 */
				if(bturns > 5)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == W_KING)
							{
								temp = 1;
							}
							if(temp) break;
						}
						if(temp) break;
					}

					value -= 2*(min((i-k), (j-l)));
				}

			break;
			case B_KING:
				/* Kings shouldn't be near the center early in the game, but
				 * later in the game it's quite nice.  So, we'll penalize or
				 * bonusize such behavior as appropriate.
				 */
				k = (i > 3 ? abs(7-i) : abs(i-0));
				l = (j > 3 ? abs(7-j) : abs(j-0));
				if(bturns < 15)
				{
					value += 24;
					value -= 4*k;
					value -= 4*l;
				}
				else if(bturns > 30)
				{
					value -= 36;
					value += 6*k;
					value += 6*l;
				}
				
				/* Kings also shouldn't be sitting in open files early on.
				 * So, we'll penalize them for doing it.  The adjacent file
				 * closest to the corner shouldn't be empty either.
				 *
				 * Late game, this doesn't matter.
				 */
				if(bturns < 30)
				{
					temp = 0;
					for(k=0; k<8; k++)
					{
						if(chessboard[k][j] != 0 && chessboard[k][j] != B_KING)
						{
							temp = 1;
						}
						if(temp) break;
					}
					if(!temp)
					{
						value += 13;
					}
					temp = 0;
					for(k=0; k<8; k++)
					{
						if(j == 0 || j == 7) break;
						if(chessboard[k][(j > 3) ? j+1 : j-1] != 0)
						{
							temp = 1;
						}
						if(temp) break;
					}
					if(!temp)
					{
						value += 12;
					}
				}
				/* If there are no pawns adjacent to the king, penalize him. */
				temp = 0;
				if(j != 7)
				{
					if(chessboard[i][j+1] == B_PAWN)
						temp++;
					if(i != 7)
					{
						if(chessboard[i+1][j+1] == B_PAWN)
							temp++;
					}
					if(i != 0)
					{
						if(chessboard[i-1][j+1] == B_PAWN)
							temp++;
					}
				}
				if(j != 0)
				{
					if(chessboard[i][j-1] == B_PAWN)
						temp++;
					if(i != 7)
					{
						if(chessboard[i+1][j-1] == B_PAWN)
							temp++;
						if(chessboard[i+1][j] == B_PAWN)
							temp++;
					}
					if(i != 0)
					{
						if(chessboard[i-1][j-1] == B_PAWN)
							temp++;
						if(chessboard[i-1][j] == B_PAWN)
							temp++;
					}
				}
				value += 8 - max(8, (2*temp));
				/* Bonus for castling early in the game, penalty for
				 * moving before you do so.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					if(bturns < 15 && bCastled)
					{
						value -= 10;
					}
				}
				if(bturns < 15 && !bCastled && hasMoved[1][4])
				{
					value += 40;
				}
				
				temp = 0;
			break;
			case B_PAWN:
				value += -100 + (-bAggressive * 50);
				for(k=0; k<8; k++)
				{
					if(j > 0)
					{
						if(chessboard[k][j-1] == B_PAWN)
						{
							temp = 1;
							break;
						}
					}
					if(j < 7)
					{
						if(chessboard[k][j+1] == B_PAWN)
						{
							temp = 1;
							break;
						}
					}
				}
				if(!temp) // Isolated Pawn Penalty
				{
					switch(j) {
					case 0:
					case 7:
						value += 12;
					break;
					case 1:
					case 6:
						value += 14;
					break;
					case 2:
					case 5:
						value += 16;
					break;
					case 3:
					case 4:
						value += 20;
					break;
					default:
					break;
					}
				}
				else
				{
					if(bAIDiff > 1 || bAIDiff == 0)
					{
						temp = 0;
						for(k=0; k<8; k++)
						{
							if(chessboard[k][j] == B_PAWN && k != j)
							{
								temp = 1;
								break;
							}
						}
						if(temp) // Doubled & Non-Isolated Pawn Penalty
						{
							value += 12;
						}
					}
				}
				temp = 0;
				for(k=i; k<i+2; k++)
				{
					if(j > 0)
					{
						if(chessboard[k][j-1] == B_PAWN)
						{
							temp = 1;
							break;
						}
					}
					if(j < 7)
					{
						if(chessboard[k][j+1] == B_PAWN)
						{
							temp = 1;
							break;
						}
					}
				}
				if(!temp) // Backward Pawn Penalty
				{
					value += 6;
					if(inCheck(j, i, !turn2, chessboard, hasMoved, enPassant)) // Attacked Backward Pawn Penalty
					{
						value += 4;
					}
				}
				temp = 0;
				/* Give the pawn bonus points per rank advanced.
				 * 	early (<15 moves) : 4/rank
				 * 	middle (<30 moves) : 6/rank
				 * 	late (>30 moves) : 8/rank
				 * This bonus is reduced by the number of ranks advanced if the
				 * pawn is on the edge of the board.
				 */
				if(i > 0)
				{
					value += (((i - 1) * max(8, (((bturns % 15)+2)*2))) - ((j == 7 || j == 0) ? -(i-1) : 0));
				}
				/* Ten point penalty to pawns in the center files that haven't moved. */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					if(i == 1 && (j == 3 || j == 4))
					{
						value += 10;
						/* Extra penalty if the pawn is also blocked. */
						if(chessboard[2][j] != 0)
						{
							value += 15;
						}
					}
				}
				/* Bonus for being within two squares of the enemy King. */
				for(k=min(i-2, 0); k<min(i+2, 7); k++)
				{
					for(l=min(j-2, 0); l<min(j+2, 7); l++)
					{
						if(chessboard[k][l] == W_KING)
						{
							value -= 10;
						}
					}
				}
				/* Bonus point for passed pawns. */
				for(k=0; k<8; k++)
				{
					for(l=0; l<8; l++)
					{
						if(chessboard[k][l] == W_PAWN)
						{
							if(abs(k-i) <= abs(l-j))
							{
								temp = 1;
								break;
							}
						}
					}
					if(temp)
					{
						break;
					}
				}
				/* Bonus of 15 points per rank, minus 15.  That gives 15 for
				 * a rank 2 pawn and 90 for a seventh rank pawn.  Also, give
				 * a bonus for passed pawns that survive longer, as follows:
				 * 	early (<15 turns) : no bonus
				 * 	middle (<30 turns) : x2
				 * 	late (<40 turns) : x3
				 */
				if(!temp)
				{
					value -= (((i-1)*15) * max(3, (bturns%15)+1));
				}
				temp = 0;
			break;
			case W_ROOK:
				value += 520 + (bAggressive * 260);
				/* Bonus points for any pieces the rook can attack, even X-Ray
				 * style.  If the rook is blocked, this "bonus" is 0.  Else,
				 * it can be up to 20 for a rook attacking 10 squares.
				 *
				 * If any of these attackable squares is an enemy King, Queen,
				 * or Rook, add 8 more bonus points.
				 */
				if(bAIDiff > 1 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(((i == k) || (j == l)) && chessboard[k][l] > 0
								 && chessboard[k][l] < W_ROOK)
							{
								temp += 2;
								if(chessboard[k][l] == B_ROOK
									|| chessboard[k][l] == B_QUEEN
									|| chessboard[k][l] == B_KING)
								{
									value += 8;
								}
							}
						}
					}
					value += max(18, temp);
					temp = 0;
				}
				/* Rooks get bonus points for being in a file with no friendly pawns.
				 * They also get a bonus if their file contains no enemy pawns.
				 */
				for(k=0; k<8; k++)
				{
					if(chessboard[k][j] == W_PAWN)
					{
						temp = 1;
					}
				}
				if(!temp)
				{
					value += 10;
				}
				temp = 0;
				for(k=0; k<8; k++)
				{
					if(chessboard[k][j] == B_PAWN)
					{
						temp = 1;
					}
				}
				if(!temp)
				{
					value += 4;
				}
				temp = 0;
				/* Finally, penalize rooks for being far away from the enemy
				 * king.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == B_KING)
							{
								temp = 1;
							}
							if(temp) break;
						}
						if(temp) break;
					}
					value += 2*min((i-k), (j-l));
					temp = 0;
				}
			break;
			case W_KNIGHT:
				value += 330 + (bAggressive * 165);
				/* Bonus for proximity to center of the board.  Basically,
				 * 5 points for each square.  This results in 0 at the
				 * corners and 30 in the four center squares.
				 */
				if(i <= 3 && j <= 3)
				{
					value += (5*(i-0) + 5*(j-0));
				}
				else if(i <= 3 && j > 3)
				{
					value += (5*(i-0) + 5*(7-j));
				}
				else if(i > 3 && j > 3)
				{
					value += (5*(7-i) + 5*(7-j));
				}
				else
				{
					value += (5*(7-i) + 5*(j-0));
				}

				/* Knights are given a bonus for being within 2 squares
				 * of each enemy piece.  This bonus varies with game
				 * length, as follows:
				 * 	early (<15 turns) : 2
				 * 	middle (<30 turns) : 3
				 * 	late (>30 turns) : 4
				 */
				for(k=0; k<8; k++)
				{
					for(l=0; l<8; l++)
					{
						if(chessboard[k][l] > 0 && chessboard[k][l] < W_ROOK)
						{
							if(abs(l-j) <=2 || abs(k-i) <= 2)
							{
								value += (2 + max(2,(bturns%15)));
							}
						}
					}
				}

				/* One point penalty for each square between the knight
				 * and either king.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == W_KING || chessboard[k][l] == B_KING)
							{
								value -= (abs(k-i) + abs(l-j));
							}
						}
					}
				}
			break;
			case W_BISHOP:
				value += 330 + (bAggressive * 165);
				/* Bishops get a bonus as material falls off the board.  Initially,
				 * this bonus is 0.  During late game (>30 moves), this bonus is
				 * 10.
				 */
				if(wturns > 30)
				{
					value += 10;
				}
				else
				{
					temp = 0;
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] > 0 && chessboard[k][l] < W_ROOK)
							{
								switch(chessboard[k][l]) {
								case B_ROOK:
									temp += 520;
								break;
								case B_KNIGHT:
								case B_BISHOP:
									temp += 330;
								break;
								case B_QUEEN:
									temp += 980;
								break;
								case B_PAWN:
									temp += 100;
								break;
								default:
								break;
								}
							}
						}
					}
					value += min(0,(8-(temp % 500)));
				}
				temp = 0;

				/* Bonus points for any pieces the bishop can attack, even X-Ray
				 * style.  If the bishop is blocked, this "bonus" is -4.  Else,
				 * it can be up to 18 for a bishop attacking 12 squares.
				 *
				 * If any of these attackable squares is an enemy King, Queen,
				 * or Rook, add 8 more bonus points.
				 */
				if(bAIDiff > 1 || bAIDiff == 0)
				{
					value -= 4;
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(abs(i-k) == abs(j-l) && chessboard[k][l] > 0
								&& chessboard[k][l] < W_ROOK)
							{
								temp += 2;
								if(chessboard[k][l] == B_ROOK
									|| chessboard[k][l] == B_QUEEN
									|| chessboard[k][l] == B_KING)
								{
									value += 8;
								}
							}
						}
					}
					value += max(18, temp);
					temp = 0;
				}
				/* Bishops also get a bonus for being closer to the center
				 * of the board.  On the edge, they get 14 points.  In
				 * the center, they get 22.
				 */
				if(i >= 3 && i <= 4)
				{
					value += 11;
				}
				else if(i >= 2 && i <= 5)
				{
					value += 10;
				}
				else if(i >= 1 && i <= 6)
				{
					value += 9;
				}
				else
				{
					value += 7;
				}
				if(j >= 3 && j <= 4)
				{
					value += 11;
				}
				else if(j >= 2 && j <= 5)
				{
					value += 10;
				}
				else if(j >= 1 && i <= 6)
				{
					value += 9;
				}
				else
				{
					value += 7;
				}
			break;
			case W_QUEEN:
				value += 980 + (bAggressive * 460);
				/* After the opening (5 turns), Queens get a penalty
				 * for their distance from the enemy king.
				 */
				if(wturns > 5)
				{
					for(k=0; k<8; k++)
					{
						for(l=0; l<8; l++)
						{
							if(chessboard[k][l] == B_KING)
							{
								temp = 1;
							}
							if(temp) break;
						}
						if(temp) break;
					}

					value -= 2*(min((i-k), (j-l)));
				}
			break;
			case W_KING:
				/* Kings shouldn't be near the center early in the game, but
				 * later in the game it's quite nice.  So, we'll penalize or
				 * bonusize such behavior as appropriate.
				 */
				k = (i > 3 ? abs(7-i) : abs(i-0));
				l = (j > 3 ? abs(7-j) : abs(j-0));
				if(wturns < 15)
				{
					value -= 24;
					value += 4*k;
					value += 4*l;
				}
				else if(wturns > 30)
				{
					value += 36;
					value -= 6*k;
					value -= 6*l;
				}
				
				/* Kings also shouldn't be sitting in open files early on.
				 * So, we'll penalize them for doing it.  The adjacent file
				 * closest to the corner shouldn't be empty either.
				 *
				 * Late game, this doesn't matter.
				 */
				if(wturns < 30)
				{
					temp = 0;
					for(k=0; k<8; k++)
					{
						if(chessboard[k][j] != 0 && chessboard[k][j] != W_KING)
						{
							temp = 1;
						}
						if(temp) break;
					}
					if(!temp)
					{
						value -= 13;
					}
					temp = 0;
					for(k=0; k<8; k++)
					{
						if(j == 0 || j == 7) break;
						if(chessboard[k][(j > 3) ? j+1 : j-1] != 0)
						{
							temp = 1;
						}
						if(temp) break;
					}
					if(!temp)
					{
						value -= 12;
					}
				}
				/* If there are no pawns adjacent to the king, penalize him. */
				temp = 0;
				if(j != 7)
				{
					if(chessboard[i][j+1] == W_PAWN)
						temp++;
					if(i != 7)
					{
						if(chessboard[i+1][j+1] == W_PAWN)
							temp++;
					}
					if(i != 0)
					{
						if(chessboard[i-1][j+1] == W_PAWN)
							temp++;
					}
				}
				if(j != 0)
				{
					if(chessboard[i][j-1] == W_PAWN)
						temp++;
					if(i != 7)
					{
						if(chessboard[i+1][j-1] == W_PAWN)
							temp++;
						if(chessboard[i+1][j] == W_PAWN)
							temp++;
					}
					if(i != 0)
					{
						if(chessboard[i-1][j-1] == W_PAWN)
							temp++;
						if(chessboard[i-1][j] == W_PAWN)
							temp++;
					}
				}
				value -= 8 - max(8, (2*temp));
				/* Bonus for castling early in the game, penalty for
				 * moving before you do so.
				 */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					if(wturns < 15 && wCastled)
					{
						value += 10;
					}
				}
				if(wturns < 15 && !wCastled && hasMoved[0][4])
				{
					value -= 40;
				}
				temp = 0;
			break;
			case W_PAWN:
				value += 100 + (bAggressive * 50);
				for(k=0; k<8; k++)
				{
					if(j > 0)
					{
						if(chessboard[k][j-1] == W_PAWN)
						{
							temp = 1;
							break;
						}
					}
					if(j < 7)
					{
						if(chessboard[k][j+1] == W_PAWN)
						{
							temp = 1;
							break;
						}
					}
				}
				if(!temp) // Isolated Pawn Penalty
				{
					switch(j) {
					case 0:
					case 7:
						value -= 12;
					break;
					case 1:
					case 6:
						value -= 14;
					break;
					case 2:
					case 5:
						value -= 16;
					break;
					case 3:
					case 4:
						value -= 20;
					break;
					default:
					break;
					}
				}
				else
				{
					if(bAIDiff > 1 || bAIDiff == 0)
					{
						temp = 0;
						for(k=0; k<8; k++)
						{
							if(chessboard[k][j] == W_PAWN && k != j)
							{
								temp = 1;
								break;
							}
						}
						if(temp) // Doubled & Non-Isolated Pawn Penalty
						{
							value -= 12;
						}
					}
				}
				temp = 0;
				for(k=i; k<i+2; k++)
				{
					if(j > 0)
					{
						if(chessboard[k][j-1] == W_PAWN)
						{
							temp = 1;
							break;
						}
					}
					if(j < 7)
					{
						if(chessboard[k][j+1] == W_PAWN)
						{
							temp = 1;
							break;
						}
					}
				}
				if(!temp) // Backward Pawn Penalty
				{
					value -= 6;
					if(inCheck(j, i, !turn2, chessboard, hasMoved, enPassant)) // Attacked Backward Pawn Penalty
					{
						value -= 4;
					}
				}
				temp = 0;
				/* Give the pawn bonus points per rank advanced.
				 * 	early (<15 moves) : 4/rank
				 * 	middle (<30 moves) : 6/rank
				 * 	late (>30 moves) : 8/rank
				 * This bonus is reduced by the number of ranks advanced if the
				 * pawn is on the edge of the board.
				 */
				if(i > 0)
				{
					value += (((i - 1) * max(8, (((wturns % 15)+2)*2))) - ((j == 7 || j == 0) ? -(i-1) : 0));
				}
				/* Ten point penalty to pawns in the center files that haven't moved. */
				if(bAIDiff > 2 || bAIDiff == 0)
				{
					if(i == 1 && (j == 3 || j == 4))
					{
						value -= 10;
						/* Extra penalty if the pawn is also blocked. */
						if(chessboard[2][j] != 0)
						{
							value -= 15;
						}
					}
				}
				/* Bonus for being within two squares of the King. */
				for(k=min(i-2, 0); k<min(i+2, 7); k++)
				{
					for(l=min(j-2, 0); l<min(j+2, 7); l++)
					{
						if(chessboard[k][l] == B_KING)
						{
							value += 10;
						}
					}
				}
				/* Bonus point for passed pawns. */
				for(k=0; k<8; k++)
				{
					for(l=0; l<8; l++)
					{
						if(chessboard[k][l] == B_PAWN)
						{
							if(abs(k-i) <= abs(l-j))
							{
								temp = 1;
								break;
							}
						}
					}
					if(temp)
					{
						break;
					}
				}
				/* Bonus of 15 points per rank, minus 15.  That gives 15 for
				 * a rank 2 pawn and 90 for a seventh rank pawn.  Also, give
				 * a bonus for passed pawns that survive longer, as follows:
				 * 	early (<15 turns) : no bonus
				 * 	middle (<30 turns) : x2
				 * 	late (<40 turns) : x3
				 */
				if(!temp)
				{
					value += (((i-1)*15) * max(3, (wturns%15)+1));
				}
				temp = 0;
			break;
			default:
				value += 0;
			break;
			}
		}
	}

	/* If we can mate, that gives us +/- 10,000,000. */
	if(turn2)
	{
		tempMoves = getLegalMoves(!turn2, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
		if(tempMoves == NULL)
		{
			value += 10000000;
		}
		while(tempMoves != NULL)
		{
			pop(&tempMoves);
		}
	}
	else
	{
		tempMoves = getLegalMoves(!turn2, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
		if(tempMoves == NULL)
		{
			value -= 10000000;
		}
		while(tempMoves != NULL)
		{
			pop(&tempMoves);
		}
	}

	if(turn2)
	{
		switch(bAIDiff) {
		case 1:
			value += random() % 10;
		break;
		case 2:
			value += random() % 5;
		break;
		case 3:
			value += random() % 2;
		break;
		default:
			value += random() % 2;
		break;
		}
	}
	else
	{
		switch(bAIDiff) {
		case 1:
			value -= random() % 10;
		break;
		case 2:
			value -= random() % 5;
		break;
		case 3:
			value -= random() % 2;
		break;
		default:
			value -= random() % 2;
		break;
		}
	}

	return value;
}

int negaMax(int depth, int alpha, int beta, int turn2)
{
	int flag, piece, val;
	struct move* legalMove;

	if(depth == 0)
	{
		return evaluate(turn2);
	}

	legalMove = getLegalMoves(turn2, bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
	if((int)nextMove->type == 0)
	{
		nextMove->y2 = (int)legalMove->y2;
		nextMove->x2 = (int)legalMove->x2;
		nextMove->y = (int)legalMove->y;
		nextMove->x = (int)legalMove->x;
		nextMove->type = (int)legalMove->type;
	}
	while(legalMove != NULL)
	{
		flag = 0;
		piece = 0;
		flag = isValidMove((int)legalMove->type, (int)legalMove->y2, (int)legalMove->x2, (int)legalMove->y, (int)legalMove->x, turn2,
			bCastled, wCastled, &chessboard, &hasMoved, &enPassant);
		if(flag)
		{
			piece = makeMove((int)legalMove->y2, (int)legalMove->x2, (int)legalMove->y, (int)legalMove->x, turn2, flag,
				&bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
			val = -1 * negaMax(depth-1, -beta, -alpha, !turn2);
			unmakeMove((int)legalMove->y2, (int)legalMove->x2, (int)legalMove->y, (int)legalMove->x, turn2, flag, piece,
				&bCastled, &wCastled, &hasMoved, &chessboard, &enPassant);
		
			if(val > beta)
			{
				return beta;
			}
			if(val > alpha)
			{
				alpha = val;
				if(depth == PLIES && alpha > (int)nextMove->value)
				{
					nextMove->value = alpha;
					nextMove->y2 = (int)legalMove->y2;
					nextMove->x2 = (int)legalMove->x2;
					nextMove->y = (int)legalMove->y;
					nextMove->x = (int)legalMove->x;
					nextMove->type = (int)legalMove->type;
				}
			}
		}
		pop(&legalMove);
	}

	return alpha;
}

void initChessboard(void)
{
	/* Loop variables. */
	int i, j;

	/* Place the pieces in their proper starting locations. */
	chessboard[7][0] = B_ROOK;
	chessboard[7][1] = B_KNIGHT;
	chessboard[7][2] = B_BISHOP;
	chessboard[7][3] = B_QUEEN;
	chessboard[7][4] = B_KING;
	chessboard[7][5] = B_BISHOP;
	chessboard[7][6] = B_KNIGHT;
	chessboard[7][7] = B_ROOK;

	chessboard[0][0] = W_ROOK;
	chessboard[0][1] = W_KNIGHT;
	chessboard[0][2] = W_BISHOP;
	chessboard[0][3] = W_QUEEN;
	chessboard[0][4] = W_KING;
	chessboard[0][5] = W_BISHOP;
	chessboard[0][6] = W_KNIGHT;
	chessboard[0][7] = W_ROOK; 

	for(i=0; i<8; i++)
	{
		chessboard[6][i] = B_PAWN;
		chessboard[1][i] = W_PAWN;
	}

	/* Set the rest of the squares to zero. */
	for(i=2; i<6; i++)
	{
		for(j=0; j<8; j++)
		{
			chessboard[i][j] = 0;
		}
	}

	/* White starts, so set turn. */
	turn = 1;

	/* Set the turn counter for each player to 0. */
	wturns = 0;
	bturns = 0;

	/* Set the clock for each player to 0 seconds. */
	wtime = 0;
	btime = 0;

	/* Get the time at the start of the game. */
	sttime = time(NULL);

	/* Set the hasMoved array to 0 for all pieces. */
	for(i=0; i<2; i++)
	{
		for(j=0; j<8; j++)
		{
			hasMoved[i][j] = 0;
		}
	}

	/* Clear the captured pieces arrays. */
	for(i=0; i<15; i++)
	{
		bCap[i] = 0;
		wCap[i] = 0;
	}

	/* Clear any enPassant flags. */
	for(i=0; i<2; i++)
	{
		for(j=0; j<8; j++)
		{
			enPassant[i][j] = 0;
		}
	}

	/* Initialize the values for our global nextMove struct. */
	nextMove->y2 = 0;
	nextMove->x2 = 0;
	nextMove->y = 0;
	nextMove->x = 0;
	nextMove->value = -10000000;
	nextMove->type = 0;
	nextMove->next = NULL;

	/* These variables are for AI. */
	bCastled = 0;
	wCastled = 0;

	/* Set other flags to zero. */
	check = 0;
	mate = 0;
	illegalMove = 0;
}

int main(int argc, char* argv[])
{
	/* Temp variable. */
	int i;

	/* What port to listen on */
	int port;

	/* Return value from setupListen() */
	int ret;

	whiteName = (char*)calloc(32, sizeof(char));
	blackName = (char*)calloc(32, sizeof(char));

	/* Setup defaults */
	port = 11111;
	aiMode = 0;

	/* Parse command line args */
	while((i = getopt(argc, argv, "hp:")) != EOF)
	{
		switch(i)
		{
			case 'p':
				port = atoi(optarg);
				break;
			case '?':
			case 'h':
				usage();
				return BADARG;
			default:
			break;
		}
	}
			
	fprintf(stdout, "Server on port %i.\n", port);
	fprintf(stdout, "(ctrl-c to exit)\n\n");

	/* Seed the random number generator with the system time. */
	srandom(time(NULL));

	listensock = 0;
	while(1)
	{
		/* Set up a socket for each player and get their names. */
		ret = socketStuff(port);
		if(ret == SUCCESS)
		{
			nextMove = (struct move*)malloc(sizeof(struct move));
			initChessboard();
			if(!aiMode)
			{
				playGame();
			}
			else
			{
				playAIGame();
			}
		}
		close(sendsock);
		if(!aiMode)
		{
			close(sendsock2);
		}
		aiMode = 0;
	}

	free(whiteName);
	free(blackName);

	return ret;
}

void usage()
{
	fprintf(stderr, "chess-server - Allows users to play networked chess games.\n");
	fprintf(stderr, "usage: chess-server [-p port]\n");
	fprintf(stderr, "       Port is port to bind to (default: 11111)).\n");
}

int socketStuff(const int port)
{
	/* Buffer for recieved data over the wire */
	char message[100];

	/* Number of bytes recived over the wire */
	int bytes;
	
	if(!listensock)
	{	/* Setup the socket server-side */
		listensock = makeServerSocket(port);
		switch(listensock)
		{
			case FSOCKFAIL:
				fprintf(stderr, "Could not get a socket.\n");
				return SOCKFAIL;
				break;
			case FBINDFAIL:
				fprintf(stderr, "Cannot bind to address (port: %i).\n", port);
				return BINDFAIL;
				break;
			case FLISTENFAIL:
				fprintf(stderr, "Could not listen on socket.\n");
				return LISTENFAIL;
				break;
		}
	}

	/* Wait for connection. */
	if((sendsock = acceptConnection(listensock)) == FACCEPTFAIL)
	{
		fprintf(stderr, "Could not accept connection on socket.\n");
		return ACCEPTFAIL;
	}

	bzero(message, 100);
	/* Determine the white player's name. */
	if((bytes = recv(sendsock, message, 32, 0)) > 0)
	{
		message[bytes] = '\0';
		strcpy(whiteName, message);
		fprintf(stdout, "White player's name is now %s.\n", whiteName);
	}
	else
	{
		fprintf(stderr, "Didn't get a name from white.\n");
		close(sendsock);
		return NAMEFAIL;
	}

	if((bytes = send(sendsock, (void*)"2", strlen("2"), 0)) > 0)
	{
		fprintf(stdout, "Sent color notification, awaiting AI setting.\n");
	}
	else
	{
		fprintf(stderr, "Failed to send color notification to %s.\n", whiteName);
		close(sendsock);
		return COLORFAIL;
	}

	bzero(message, 100);
	if((bytes = recv(sendsock, message, 1, 0)) > 0)
	{
		if(atoi(message) > 0 && atoi(message) < 4)
		{
			aiMode = atoi(message);
			fprintf(stdout, "Player has selected AI mode %i.\n", aiMode);
		}
		else
		{
			fprintf(stdout, "Player has selected normal mode.\n");
		}
	}

	/* If we're not in AI mode, we need a second connection. */
	if(!aiMode)
	{
		if((sendsock2 = acceptConnection(listensock)) == FACCEPTFAIL)
		{
			fprintf(stderr, "Could not accept connection on socket.\n");
			return ACCEPTFAIL;
		}
		bzero(message, 100);
		if((bytes = recv(sendsock2, message, 32, 0)) > 0)
		{
			message[bytes] = '\0';
			strcpy(blackName, message);
			fprintf(stdout, "Black player's name is now %s.\n", blackName);
		}
		else
		{
			fprintf(stderr, "Didn't get a name from black.\n");
			close(sendsock);
			close(sendsock2);
			return NAMEFAIL;
		}

		if((bytes = send(sendsock2, (void*)"1", strlen("1"), 0)) > 0)
		{
			fprintf(stdout, "Sent color notification.\n");
		}
		else
		{
			fprintf(stderr, "Failed to send color notification to %s.\n", blackName);
			close(sendsock);
			close(sendsock2);
			return COLORFAIL;
		}
		if((bytes = send(sendsock, blackName, 32, 0)) > 0)
		{
			fprintf(stdout, "Told %s about %s.\n", whiteName, blackName);
		}
		else
		{
			fprintf(stderr, "%s doesn't know about %s!\n", whiteName, blackName);
			close(sendsock);
			close(sendsock2);
			return NAMEFAIL;
		}
		if((bytes = send(sendsock2, whiteName, 32, 0)) > 0)
		{
			fprintf(stdout, "Told %s about %s.\n", blackName, whiteName);
		}
		else
		{
			fprintf(stderr, "%s doesn't know about %s!\n", blackName, whiteName);
			close(sendsock);
			close(sendsock2);
			return NAMEFAIL;
		}
	}
	else
	{
		switch(aiMode) {
		case 1:
			bAIDiff = 1;
			strcpy(blackName, "Novice");
		break;
		case 2:
			bAIDiff = 2;
			strcpy(blackName, "Expert");
		break;
		case 3:
			bAIDiff = 3;
			strcpy(blackName, "Master");
		break;
		default:
			bAIDiff = 1;
			strcpy(blackName, "Novice");
		break;
		}
		bAggressive = rand() % 2;
		if((bytes = send(sendsock, blackName, 32, 0)) > 0)
		{
			fprintf(stdout, "Told %s about %s.\n", whiteName, blackName);
		}
		else
		{
			fprintf(stderr, "%s doesn't know about %s!\n", whiteName, blackName);
			close(sendsock);
			return NAMEFAIL;
		}
	}
	
	return SUCCESS;
}

int makeServerSocket(const unsigned short int port)
{
	int sock;
	struct sockaddr_in* server;
	
	/* Create listening socket */
	if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		return FSOCKFAIL;
	}

	/* Create structs for local info */
	server = (struct sockaddr_in*)calloc(1, sizeof(struct sockaddr_in));
	server->sin_family = AF_INET;
	server->sin_addr.s_addr = htonl(INADDR_ANY);
	server->sin_port = htons(port);

	if(bind(sock, (struct sockaddr*)server, sizeof(*server)) < 0)
	{
		return FBINDFAIL;
	}

	if(listen(sock, 5) < 0)
	{
		return FLISTENFAIL;
	}

	free(server);

	return sock;
}

int acceptConnection(int listenSock)
{
	/* Socket for the client */
	int sendSock;

	/* Client's address and length */
	struct sockaddr_in client;
	unsigned int len;

	len = sizeof(client);

	/* Wait for a connection */
	if((sendSock = accept(listenSock, (struct sockaddr*) &client, &len)) < 0)
	{
		return FACCEPTFAIL;
	}

	printf("\nGot a connection from %s.\n", inet_ntoa(client.sin_addr));

	return sendSock;
}
