#ifndef SERVER_H
#define SERVER_H

/* For printf() */
#include <stdio.h>

/* For *alloc(), atoi() */
#include <stdlib.h>

/* For strlen() */
#include <string.h>

/* For most everything */
#include <unistd.h>

/* For socket stuff */
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

/* For time() */
#include <time.h>

/* Main return codes */
#define SUCCESS 0

#define BADARG 1
#define SOCKFAIL 10
#define BINDFAIL 11
#define LISTENFAIL 12
#define ACCEPTFAIL 13

/* function return codes */
#define FSOCKFAIL -1
#define FBINDFAIL -2 
#define FLISTENFAIL -3
#define FACCEPTFAIL -4
#define NAMEFAIL -5
#define COLORFAIL -6

/* Plays a game of chess with two human players. */
void playGame(void);

/* Plays a game with a human player and an AI player. */
void playAIGame(void);

/* Parses a move sent to the server from a client.
 * Moves are in the form a1:a1 where the 'a' is the
 * file and the '1' is the rank and the first 'a1' is
 * the starting positions while the second is the
 * destination.
 */
void parseMove(char* buffer);

/* For AI. */
int evaluate(int turn2);
int negaMax(int depth, int alpha, int beta, int turn2);

/* Sets up a chessboard and any variables that the server needs
 * to track.
 */
void initChessboard(void);

/**
 * Prints the usage information about the program
 */
void usage(void);

/**
 * Does most everything, infinite loop, those things
 *
 * @param port The port to listen on
 * @param pass The password for authentification
 * @return @see main return codes
 */
int socketStuff(const int port);

/**
 * Makes a server-side socket
 *
 * @param port The port to listen on
 * @return A descriptor referencing the new socket, or error
 */
int makeServerSocket(const unsigned short int port);

/**
 * Accepts a connection on a socket
 *
 * @param listensock The socket to accept the connection on
 * @return A descriptor referenecing the accepted connection, or error
 */
int acceptConnection(int listensock);

#endif /* SERVER_H */
